/*****************************************************************************
 *  Rohde&Schwarz RTM Instrument Driver                               
 *  LabWindows/CVI Instrument Driver                                     
 *  Original Release: 2010-12-17                                 
 *  By: Jiri Kominek, Martin Krcmar
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       See ChangeLog
 *                                                                           
 *****************************************************************************/

#include <string.h> 
#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include "rsrtm20xx.h" 
#if defined (_CVI_)
	#include <utility.h>
#endif

ViStatus _VI_FUNC  rsrtm20xx_RsClose (ViSession vi);

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/

 static ViString signalSourceArr[] = {"NONE","CH1","CH2","CH3","CH4","MA1","MA2","MA3","MA4","MA5",
										"R1","R2","R3","R4","TRIG","D1","D2","D3","D4",
										"D5","D6","D7","D8","D9","D10","D11","D12","D13",
										"D14","D15","D70","D158","D0","SPEC", "MINH", "MAXH", "AVER", VI_NULL};
 static ViString languageArr[] = {"ENGL","GERM","FREN","SPAN","RUSS","SCH","TCH","JAP","KOR",VI_NULL};
 static ViString statisticsArr[] = {"ACT","AVG","STDD","NPE","PPE","WFMC",VI_NULL};
 static ViString qualityTypeArr[] = {"RMS","FREQ","CRES",VI_NULL};
 static ViString qualityTypePowerArr[] = {"REAL","REAC","APP","PFAC","PHAS",VI_NULL};
 static ViString harmonicsTypeArr[] = {"VCO","FREQ","LEV","MIN","MAX","MEAN",
															"LEV:LIM","WFMC",VI_NULL};
 static ViString rippleTypeArr[] = {"FREQ","PEAK","STDD","UPE","LPE","MEAN", "PER",
																"PDCY","NDCY",VI_NULL};
 static ViString spectrumTypeArr[] = {"FREQ","LEV","MIN","MAX","MEAN","WFMC",VI_NULL};
 static ViString slewRateTypeArr[] = {"LPE","UPE",VI_NULL}; 
 static ViString modulationTypeArr[] = {"LPE","UPE","MEAN","RMS","STDD",VI_NULL};
 static ViString efficiencyTypeArr[] = {"EFF","INP:REAL","OUTP:REAL",VI_NULL}; 
/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsrtm20xx_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsrtm20xx_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsrtm20xx_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
    {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
    }

    checkErr( rsrtm20xx_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsrtm20xx_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsrtm20xx_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    
    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));
    
    ATTRLIST(attrList,rsrtm20xx);  
    checkErr( Rs_SpecificDriverNew (instrSession, "rsrtm20xx", newOptionString, attrList));
    
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsrtm20xx);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_SESSION, 0, &instrSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsrtm20xx_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsrtm20xx_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSRTM20XX_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSRTM20XX_ATTR_ID_QUERY_RESPONSE,
                                   0, RSRTM20XX_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSRTM20XX_VALID_ID_RESPONSE_STRING, strlen(RSRTM20XX_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    
    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (instrSession) session. */
    checkErr( rsrtm20xx_CheckStatus (instrSession));

Error: 

    if (error < VI_SUCCESS)
    {
        if (instrSession != VI_NULL)
        {
            (void) viClose (instrSession);
            (void) Rs_Dispose (instrSession);
        }
        
        if (rmSession != VI_NULL)
            (void) viClose (rmSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;

} 

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the measurement.
/// HIPAR arraySize/Pass the number of elements in 'Results' parameter.
/// HIPAR results/All values from the statistics buffer.
ViStatus _VI_FUNC rsrtm20xx_QueryAutomaticMeasurementAllStatisticalValues (ViSession instrSession,
                                                                       ViInt32 measurement,
                                                                       ViInt32 arraySize,
                                                                       ViReal64 results[])
{
	ViStatus	error = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViChar *extBuf=VI_NULL;
    ViChar *pbuffer;
    ViInt32  i = 0;

	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, measurement, RSRTM20XX_VAL_MEASUREMENT_1, RSRTM20XX_VAL_MEASUREMENT_4), 2, "Measurement");

    viCheckErr (viPrintf (instrSession, "MEAS%ld:STAT:VAL:ALL?\n", measurement));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &extBuf, &retCnt)); 

    pbuffer = strtok(extBuf, ",");
    i=0;
    while (pbuffer) 
    {
        if (i < arraySize)
        {
            results[i++] = atof (pbuffer);
        }   
        pbuffer = strtok(NULL, ",");        
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    if (extBuf) free(extBuf);
    return error;
	
}

/// HIFN  This function returns the sample mode the oscilloscope is currently 
/// HIFN  using.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR sampleMode/Returns the method of adding waveform points to the samples of the ADC 
/// HIPAR sampleMode/in order to fill the record length.
ViStatus _VI_FUNC rsrtm20xx_SampleMode(
    ViSession   instrSession,
    ViInt32*    sampleMode
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_GetAttributeViInt32 (instrSession, "",
                                               RSRTM20XX_ATTR_SAMPLE_MODE,
                                               sampleMode), 2, "SampleMode");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function performs an autoset process: analyzes the enabled channel signals, and obtains appropriate horizontal, vertical, and trigger settings to display stable waveforms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
ViStatus _VI_FUNC rsrtm20xx_Autoset(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr(rsrtm20xx_SetAttributeViString(instrSession, "",RSRTM20XX_ATTR_AUTOSCALE, VI_NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures how the oscilloscope acquires data and fills the waveform record.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR acquisitionType/Specify the manner in which you want the oscilloscope to acquire data and fill the waveform record.
ViStatus _VI_FUNC rsrtm20xx_ConfigureAcquisitionType(
    ViSession instrSession,
    ViInt32   channel,
    ViInt32   acquisitionType
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_ACQUISITION_TYPE,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_ACQUISITION_TYPE, acquisitionType), 4, "Acquisition Type");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the common attributes of the acquisition subsystem.  These attributes are the time per record and acquisition start time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR timePerRecord/Pass the time in seconds that corresponds to the record length.  
/// HIPAR acquisitionStartTime_Offset/Specifies the length of time from the trigger event to the first point in the waveform record.  If the value is positive, the first point in the waveform record occurs after the trigger event.  If the 
///           value is negative, the first point in the waveform record occurs before the trigger event.
ViStatus _VI_FUNC rsrtm20xx_ConfigureAcquisitionRecord(
    ViSession   instrSession,
    ViReal64    timePerRecord,
    ViReal64    acquisitionStartTime_Offset
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, "",
        RSRTM20XX_ATTR_HORZ_TIME_PER_RECORD, timePerRecord), 2, "Time Per Record");
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, "",
        RSRTM20XX_ATTR_ACQUSITION_START_TIME, acquisitionStartTime_Offset), 3, "Acqusition Start Time");  
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the interpolation method the oscilloscope uses when it cannot sample a voltage for a point in the waveform record.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR interpolation/Pass the interpolation method you want the oscilloscope to 
ViStatus _VI_FUNC rsrtm20xx_ConfigureInterpolation(
    ViSession   instrSession,
    ViInt32     interpolation
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
        RSRTM20XX_ATTR_INTERPOLATION, interpolation), 2, "Interpolation");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the number of waveforms that the oscilloscope  acquires and averages.  After the oscilloscope acquires as many waveforms as you specify, it returns to the idle state.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR numberofAverages/Pass the number of waveforms you want the oscilloscope to acquire and to average. After the oscilloscope acquires as many waveforms as you specify, it returns to the idle state.
ViStatus _VI_FUNC rsrtm20xx_ConfigureNumberOfAverages(
    ViSession    instrSession,
    ViInt32      numberofAverages
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
        RSRTM20XX_ATTR_NUM_AVERAGES, numberofAverages), 2, "Number of Averages");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
} 


/// HIFN  This function configures the number of waveforms acquired.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR numberOfWaveforms/Sets the number of waveforms acquired
ViStatus _VI_FUNC rsrtm20xx_ConfigureNumberOfWaveforms (ViSession instrSession,
                                                    ViInt32 numberOfWaveforms)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
   
    viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, "",
        RSRTM20XX_ATTR_NUMBER_OF_WAVEFORMS, numberOfWaveforms), 2, "Number Of Waveforms");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the common attributes of the oscilloscope's channel subsystem for a particular channel.  These attributes are the vertical range, vertical offset, coupling, probe attenuation, 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR verticalRange/Sets the voltage range across the 10 vertical divisions of the diagram.
/// HIPAR verticalOffset/Pass the location of the center of the range that you specify with for the Vertical Range parameter. Express the value with respect to ground.
/// HIPAR verticalCoupling/Specify how you want the oscilloscope to couple the input signal for the channel.
/// HIPAR channelEnabled/Switches the channel signal on or off.
ViStatus _VI_FUNC rsrtm20xx_ConfigureChannel(
    ViSession    instrSession,
    ViInt32      channel,
    ViBoolean    channelEnabled,
    ViReal64     verticalRange,
    ViReal64     verticalOffset,
    ViInt32      verticalCoupling
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_CHANNEL_ENABLED,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm20xx_SetAttributeViBoolean(instrSession, rep_cap,
        RSRTM20XX_ATTR_CHANNEL_ENABLED, channelEnabled), 3, "Channel Enabled");
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_VERTICAL_RANGE, verticalRange), 4, "Vertical Range");
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_VERTICAL_OFFSET, verticalOffset), 5, "Vertical Offset"); 

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_VERTICAL_COUPLING, verticalCoupling), 6, "Vertical Coupling");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the bandwidth limit for the indicated channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bandwidth/Selects the bandwidth limit for the indicated channel.
ViStatus _VI_FUNC rsrtm20xx_ConfigureChannelBandwidth(
    ViSession    instrSession,
    ViInt32      channel,
    ViInt32      bandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_CHANNEL_BANDWIDTH_LIMIT,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_CHANNEL_BANDWIDTH_LIMIT, bandwidth), 3, "Bandwidth");  
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the probe characteristics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Name 
/// HIPAR bufferSize/and Attenuation values.
/// HIPAR type/Queries the type of the probe.
/// HIPAR name/Queries the name of the probe.
/// HIPAR inputImpedance/Queries the termination of the probe.
/// HIPAR inputCapacitance/Queries the input capacitance of the probe.
/// HIPAR attenuation/Queries the attenuation of the probe.
/// HIPAR bandwidth/Queries the bandwidth of the probe.
ViStatus _VI_FUNC rsrtm20xx_QueryProbeCharacteristics(
    ViSession    instrSession,
    ViInt32        channel,
    ViInt32        bufferSize,
    ViChar        type[],
    ViChar        name[],
    ViInt32*    inputImpedance,
    ViReal64*    inputCapacitance,
    ViReal64*    attenuation,
    ViReal64*    bandwidth
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, channel, RSRTM20XX_VAL_CHANNEL_1, RSRTM20XX_VAL_CHANNEL_4), 2, "Channel");

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_PROBE_TYPE,
                                                 "Channel",
                                                 channel,
                                                 10,
                                                 rep_cap), 2, "Channel"); 
    
    viCheckParm (rsrtm20xx_GetAttributeViString (instrSession, rep_cap,
                                               RSRTM20XX_ATTR_PROBE_TYPE,
                                               bufferSize,
                                               type), 4, "Type");
    
    viCheckParm (rsrtm20xx_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_PROBE_NAME,
                                                bufferSize,
                                                name), 5, "Name");
    
    viCheckParm (rsrtm20xx_GetAttributeViInt32 (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_PROBE_INPUT_IMPEDANCE,
                                                inputImpedance), 6, "Input Impedance");
    
    viCheckParm (rsrtm20xx_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_PROBE_INPUT_CAPACITANCE,
                                                inputCapacitance), 7, "Input Capacitance");
    
    viCheckParm (rsrtm20xx_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_PROBE_ATTENUATION,
                                                attenuation), 8, "Attenuation");    
    
    viCheckParm (rsrtm20xx_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_PROBE_BANDWIDTH,
                                                bandwidth), 9, "Bandwidth");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function sets the attenuation or gain of the probe if the probe was not detected by the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR attenuation/Sets the attenuation or gain of the probe if the probe was not detected by the instrument.
ViStatus _VI_FUNC rsrtm20xx_ConfigureProbeAttenuationManual (ViSession instrSession,
                                                         ViInt32 channel,
                                                         ViReal64 attenuation)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, channel, RSRTM20XX_VAL_CHANNEL_1, RSRTM20XX_VAL_CHANNEL_4), 2, "Channel");
  
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_PROBE_ATTENUATION_MANUAL,
                                                 "Channel",
                                                 channel,
                                                 15,
                                                 rep_cap), 2, "Channel");

    viCheckParm (rsrtm20xx_SetAttributeViReal64 (instrSession, rep_cap,
        RSRTM20XX_ATTR_PROBE_ATTENUATION_MANUAL, attenuation), 3, "Attenuation"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function turns an additional 5 kHz or 100 MHz low-pass filter in the trigger path
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR filterType/Selects the filter type.
/// HIPAR filterState/Activates or deactivates a low-pass filter in the trigger path.
ViStatus _VI_FUNC rsrtm20xx_ConfigureEdgeTriggerFilter (ViSession instrSession,
                                                   ViInt32 filterType,
                                                   ViBoolean filterState)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, filterType, RSRTM20XX_VAL_TRIGGER_FILTER_LOW, 
										RSRTM20XX_VAL_TRIGGER_FILTER_HIGH), 2, "Filter Type");
    
	if (filterType == RSRTM20XX_VAL_TRIGGER_FILTER_LOW)
    {
		viCheckParm(rsrtm20xx_SetAttributeViBoolean (instrSession, "",
	        RSRTM20XX_ATTR_TRIGGER_FILTER_LOW, filterState), 4, "Filter State");    
    }
	else
	{
		viCheckParm(rsrtm20xx_SetAttributeViBoolean (instrSession, "",
	        RSRTM20XX_ATTR_TRIGGER_FILTER_HIGH, filterState), 4, "Filter State");  
	}
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures selected trigger level.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_RUNT_TRIGGER_POLARITY
/// HIFN     RSRTM20XX_ATTR_RUNT_TRIGGER_LOWER_LEVEL
/// HIFN     RSRTM20XX_ATTR_RUNT_TRIGGER_UPPER_LEVEL
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     TRIGger:A:RUNT:POLarity
/// HIFN     TRIGger:A:LEVel<Channel>:RUNT:LOWer
/// HIFN     TRIGger:A:LEVel<Channel>:RUNT:UPPer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR polarity/Sets the polarity of the pulse.
/// HIPAR channel/Sets the trigger source for the selected trigger type.
/// HIPAR lowerLevel/Set the lower voltage threshold. The instrument triggers if the 
/// HIPAR lowerLevel/amplitude crosses the first threshold twice in succession without 
/// HIPAR lowerLevel/crossing the second one.
/// HIPAR upperLevel/Set the upper voltage threshold. The instrument triggers if the 
/// HIPAR upperLevel/amplitude crosses the first threshold twice in succession without 
/// HIPAR upperLevel/crossing the second one.
ViStatus _VI_FUNC rsrtm20xx_ConfigureRuntTrigger(
	ViSession	instrSession,
	ViInt32	polarity,
	ViInt32	channel,
	ViReal64	lowerLevel,
	ViReal64	upperLevel
)
{
 	ViStatus    error = VI_SUCCESS;
    ViChar   rep_cap[15] = "";
    

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, rep_cap,
	        RSRTM20XX_ATTR_RUNT_TRIGGER_POLARITY, polarity), 4, "Polarity");
	
	viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
	                                                 RSRTM20XX_ATTR_RUNT_TRIGGER_LOWER_LEVEL,
	                                                 "Channel",
	                                                 channel,
	                                                 15,
	                                                 rep_cap), 3, "Channel"); 
    
	viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
	        RSRTM20XX_ATTR_RUNT_TRIGGER_LOWER_LEVEL, lowerLevel), 4, "Lower Level");
	
	viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
	        RSRTM20XX_ATTR_RUNT_TRIGGER_UPPER_LEVEL, upperLevel), 4, "Upper Level");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;   
}

/// HIFN  This function sets the time the instrument waits after an A-event until it recognizes B-events.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR delayTime/Sets the time the instrument waits after an A-event until it recognizes B-events.
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerDelay (ViSession instrSession,
                                           ViReal64 delayTime)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, NULL,
        RSRTM20XX_ATTR_TRIGGER_DELAY, delayTime), 2, "Delay Time");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN Sets the coupling for the external trigger input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR externalCoupling/Sets the coupling for the external trigger input.
ViStatus _VI_FUNC rsrtm20xx_ConfigureExternalTriggerCoupling (ViSession instrSession,
                                                          ViInt32 externalCoupling)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_EXTERNAL_TRIGGER_COUPLING, externalCoupling), 2, "External Coupling");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function defines the holdoff time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR holdoffTime/Defines the holdoff time.
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerHoldoffTime (ViSession instrSession,
                                                     ViReal64 holdoffTime)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
							        RSRTM20XX_ATTR_TRIGGER_HOLDOFF_TIME, 
									holdoffTime), 2, "Holdoff Time");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function activates or deactivates the second trigger. The instrument triggers if both trigger event conditions (A and B) are fulfilled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR state/This Control activates or deactivates the second trigger.
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerBState (ViSession instrSession,
                                           ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_SetAttributeViBoolean (instrSession, NULL,
        RSRTM20XX_ATTR_TRIGGER_ENABLED, state), 2, "State");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets a number of B-trigger events that fulfill all B-trigger conditions but do not cause the trigger. The oscilloscope triggers on the n-th event (the last of the specified number of events).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR eventCount/This Control sets a number of B-trigger events.
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerEventCount (ViSession instrSession,
                                                ViInt32 eventCount)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_TRIGGER_EVENT_COUNT, eventCount), 4, "Event Count");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the common attributes of the trigger subsystem.  These attributes are the trigger type and holdoff.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerType/Sets the trigger type.
ViStatus _VI_FUNC rsrtm20xx_ConfigureTrigger(
    ViSession   instrSession,
    ViInt32     trigger,
    ViInt32     triggerType
)
{
    ViStatus    error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, trigger, RSRTM20XX_VAL_TRIGGER_A, 
										RSRTM20XX_VAL_TRIGGER_A), 2, "Trigger");
 
	viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, "",
	        RSRTM20XX_ATTR_TRIGGER_TYPE_A, triggerType), 4, "Trigger Type");    
    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the TV trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR standard/This Control sets the color television standard.
/// HIPAR polarity/This Control sets the polarity.
/// HIPAR field/Selects the frame or line.
/// HIPAR line/This Control sets an exact line number.
ViStatus _VI_FUNC rsrtm20xx_ConfigureVideoTriggerSource (ViSession instrSession,
                                                ViInt32 standard, 
                                                ViInt32 polarity,
                                                ViInt32 field, 
                                                ViInt32 line)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_ConfigureTrigger (instrSession, RSRTM20XX_VAL_TRIGGER_A, RSRTM20XX_VAL_VIDEO_TRIGGER));
    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_TV_STANDARD, standard), 2, "Standard");    
    
    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_TV_TRIGGER_POLARITY, polarity), 3, "Polarity");    
        
    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_TV_TRIGGER_EVENT, field), 4, "Field");    
    
    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_TV_TRIGGER_LINE_NUMBER, line), 5, "Line");    
                                                
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR channel/This control selects the channel.
/// HIPAR triggerSource/selects the channel.
/// HIPAR triggerLevel/This control sets the trigger threshold voltage.
/// HIPAR triggerSlope/Selects the slope.
ViStatus _VI_FUNC rsrtm20xx_ConfigureEdgeTriggerSource (ViSession instrSession,
                                            ViInt32  trigger,
											ViInt32  channel,
                                            ViInt32  triggerSource,
                                            ViReal64 triggerLevel,
                                            ViInt32  triggerSlope)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   rep_cap[15] = "";
    

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, trigger, RSRTM20XX_VAL_TRIGGER_A, 
										RSRTM20XX_VAL_TRIGGER_B), 2, "Trigger");
  
	if (trigger == RSRTM20XX_VAL_TRIGGER_A)
    {
    	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
        	RSRTM20XX_ATTR_TRIGGER_SOURCE_A, triggerSource), 4, "Trigger Source");
	
		
		viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        RSRTM20XX_ATTR_TRIGGER_SLOPE_A, triggerSlope), 6, "Trigger Slope"); 
	
		viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
	                                                 RSRTM20XX_ATTR_TRIGGER_LEVEL_A,
	                                                 "Channel",
	                                                 channel,
	                                                 15,
	                                                 rep_cap), 2, "Channel"); 
    
	    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
	        RSRTM20XX_ATTR_TRIGGER_LEVEL_A, triggerLevel), 5, "Trigger Level");
	}
	else
	{
    	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
        	RSRTM20XX_ATTR_TRIGGER_SOURCE_B, triggerSource), 4, "Trigger Source");
	
		
		viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        RSRTM20XX_ATTR_TRIGGER_SLOPE_B, triggerSlope), 6, "Trigger Slope"); 
	
		viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
	                                                 RSRTM20XX_ATTR_TRIGGER_LEVEL_B,
	                                                 "Channel",
	                                                 channel,
	                                                 15,
	                                                 rep_cap), 2, "Channel"); 
    
	    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
	        RSRTM20XX_ATTR_TRIGGER_LEVEL_B, triggerLevel), 5, "Trigger Level");
	}
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the trigger source for the selected trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR source/Selects the source.
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerSource (ViSession instrSession,
                                            ViInt32 trigger, 
                                            ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, trigger, RSRTM20XX_VAL_TRIGGER_A, 
										RSRTM20XX_VAL_TRIGGER_B), 2, "Trigger");

	if (trigger == RSRTM20XX_VAL_TRIGGER_A)
    {
	    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        		RSRTM20XX_ATTR_TRIGGER_SOURCE_A, source), 3, "Source");
	}
	else
	{
	    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        		RSRTM20XX_ATTR_TRIGGER_SOURCE_B, source), 3, "Source");
	}
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the coupling for the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR coupling/Selects coupling event for the trigger source.
ViStatus _VI_FUNC rsrtm20xx_ConfigureEdgeTriggerCoupling (ViSession instrSession,
                                                ViInt32 coupling)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, NULL,
        RSRTM20XX_ATTR_TRIGGER_COUPLING, coupling), 2, "Coupling");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the trigger mode. The trigger mode determines the behaviour of the instrument if no trigger occurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR triggerModifier/Sets the trigger mode. The trigger mode determines the behaviour of the instrument if no trigger occurs.
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerModifier(
    ViSession   instrSession,
    ViInt32     trigger,
    ViInt32     triggerModifier
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, trigger, RSRTM20XX_VAL_TRIGGER_A, 
										RSRTM20XX_VAL_TRIGGER_B), 2, "Trigger");
  
	if (trigger == RSRTM20XX_VAL_TRIGGER_A)
    {
	    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        		RSRTM20XX_ATTR_TRIGGER_MODIFIER_A, triggerModifier), 3, "Trigger Modifier");
	}
	else
	{
	    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        		RSRTM20XX_ATTR_TRIGGER_MODIFIER_B, triggerModifier), 3, "Trigger Modifier");
	}
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the trigger level automatically. The attribute is not available for an external trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
ViStatus _VI_FUNC rsrtm20xx_TriggerLevelAutoFind(
    ViSession   instrSession,
    ViInt32     trigger
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, trigger, RSRTM20XX_VAL_TRIGGER_A, 
										RSRTM20XX_VAL_TRIGGER_B), 2, "Trigger");
  
	if (trigger == RSRTM20XX_VAL_TRIGGER_A)
    {
	    viCheckParm(rsrtm20xx_SetAttributeViString (instrSession, "",
	        RSRTM20XX_ATTR_TRIGGER_LEVEL_AUTO_FIND_A, VI_NULL), 3, "Trigger");
	}
	else
	{
	    viCheckParm(rsrtm20xx_SetAttributeViString (instrSession, "",
	        RSRTM20XX_ATTR_TRIGGER_LEVEL_AUTO_FIND_B, VI_NULL), 3, "Trigger");
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures selected trigger level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR channel/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and channel.
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerChannelLevel (ViSession instrSession,
                                                        ViInt32 trigger,
                                                        ViInt32 channel,
                                                        ViReal64 triggerLevel)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   rep_cap[15] = "";
    

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, trigger, RSRTM20XX_VAL_TRIGGER_A, 
										RSRTM20XX_VAL_TRIGGER_B), 2, "Trigger");
	
	if (trigger == RSRTM20XX_VAL_TRIGGER_A)
    {
		viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
	                                                 RSRTM20XX_ATTR_TRIGGER_LEVEL_A,
	                                                 "Channel",
	                                                 channel,
	                                                 15,
	                                                 rep_cap), 3, "Channel"); 
    
	    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
	        RSRTM20XX_ATTR_TRIGGER_LEVEL_A, triggerLevel), 4, "Trigger Level");
	}
	else
	{
		viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
	                                                 RSRTM20XX_ATTR_TRIGGER_LEVEL_B,
	                                                 "Channel",
	                                                 channel,
	                                                 15,
	                                                 rep_cap), 3, "Channel"); 
    
	    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
	        RSRTM20XX_ATTR_TRIGGER_LEVEL_B, triggerLevel), 4, "Trigger Level");
	}
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets a hysteresis range around the trigger level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR trigger/Selects trigger event.
/// HIPAR hysteresis/Sets hysteresis range.
ViStatus _VI_FUNC rsrtm20xx_ConfigureTriggerHysteresis (ViSession instrSession,
                                                          ViInt32 trigger,
                                                          ViInt32 hysteresis)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, trigger, RSRTM20XX_VAL_TRIGGER_A, 
										RSRTM20XX_VAL_TRIGGER_B), 2, "Trigger");

	if (trigger == RSRTM20XX_VAL_TRIGGER_A)
    {
	    viCheckParm (rsrtm20xx_SetAttributeViInt32(instrSession, rep_cap,
	        RSRTM20XX_ATTR_TRIGGER_HYSTERESIS_A, hysteresis), 3, "Hysteresis");
    }
	else
	{
	    viCheckParm (rsrtm20xx_SetAttributeViInt32(instrSession, rep_cap,
	        RSRTM20XX_ATTR_TRIGGER_HYSTERESIS_B, hysteresis), 3, "Hysteresis");
    }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Select the state for each digital channel. Sets the logical combination of the trigger states of the channels. Sets the trigger point depending on the result of the logical combination of the channel states. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR patternSource/This control sets the state for each digital channel.
/// HIPAR patternFunction/Sets the logical combination of the trigger states of the channels.
/// HIPAR patternCondition/This Control sets the trigger point depending on the result of the logical combination of the channel states.
ViStatus _VI_FUNC rsrtm20xx_ConfigurePatternTriggerSource (ViSession instrSession,
                                                         ViString patternSource,
                                                         ViInt32 patternFunction,
                                                         ViBoolean patternCondition)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_ConfigureTrigger (instrSession, RSRTM20XX_VAL_TRIGGER_A,
                                      RSRTM20XX_VAL_PATTERN_TRIGGER));
    viCheckParm (rsrtm20xx_SetAttributeViString (instrSession, NULL,
        RSRTM20XX_ATTR_TRIGGER_PATTERN_SOURCE, patternSource), 3, "Pattern Source");    
    
    viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, NULL,
        RSRTM20XX_ATTR_TRIGGER_PATTERN_FUNCTION, patternFunction), 4, "Pattern Function");    

    viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, NULL,
        RSRTM20XX_ATTR_TRIGGER_PATTERN_POINT_ENABLED, patternCondition), 5, "Pattern Condition");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the width trigger.  A width trigger occurs 
/// HIFN  when the oscilloscope detects a positive or negative pulse with a width 
/// HIFN  between, or optionally outside, the width thresholds.  You specify the 
/// HIFN  width thresholds with the Width Low Threshold and Width High Threshold 
/// HIFN  parameters.  You specify whether the oscilloscope triggers on pulse 
/// HIFN  widths that are within or outside the width thresholds with the Width 
/// HIFN  Condition parameter.  You specify the polarity of the pulse with the 
/// HIFN  Width Polarity parameter.  The trigger does not actually occur until 
/// HIFN  the edge of a pulse that corresponds to the Width Low Threshold, Width 
/// HIFN  High Threshold, Width Condition, and Width Polarity crosses the 
/// HIFN  threshold you specify in the Trigger Level parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR width/For the ranges "Within" and "Outside", the width defines the center of 
/// HIPAR width/ a range which is defined by the limits "+/-Delta". For the ranges 
/// HIPAR width/ "Shorter" and "Longer", the width defines the maximum and minimum pulse 
/// HIPAR width/ width, respectively.
/// HIPAR widthDelta/Defines a range around the width value.
/// HIPAR widthPolarity/Sets the polarity of the pulse.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR widthCondition/Sets how the measured pulse width is compared with the given limit(s).
ViStatus _VI_FUNC rsrtm20xx_ConfigureWidthTriggerSource(
    ViSession   instrSession,
    ViInt32     triggerSource,
    ViReal64    triggerLevel,
    ViReal64    width,
    ViReal64    widthDelta,
    ViInt32     widthPolarity,
    ViInt32     widthCondition
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsrtm20xx_ConfigureTrigger (instrSession, RSRTM20XX_VAL_TRIGGER_A,
                                      RSRTM20XX_VAL_WIDTH_TRIGGER));
	
    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_TRIGGER_SOURCE_A, triggerSource), 3, "Trigger Source");
   if (triggerSource < RSRTM20XX_VAL_LINE) {
    	viCheckParm(rsrtm20xx_ConfigureTriggerChannelLevel (instrSession,
                                                    RSRTM20XX_VAL_TRIGGER_A,
                                                    triggerSource,
                                                    triggerLevel), 4, "Trigger Level");
   }
    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, NULL,
        RSRTM20XX_ATTR_WIDTH, width), 5, "Width");

    viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, NULL,
        RSRTM20XX_ATTR_WIDTH_DELTA, widthDelta), 6, "Width Delta");   

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_WIDTH_POLARITY, widthPolarity), 7, "Width Polarity");  

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_WIDTH_CONDITION, widthCondition), 8, "Width Condition");   

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function removes the displayed persistent waveform from the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_ClearPersistence (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_SetAttributeViString (instrSession, "",
                                                RSRTM20XX_ATTR_PERSISTENCE_CLEAR,
                                                VI_NULL
                                                ), 1, "Session");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function installs the specified language on the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR language/Selects the language.
/// HIPAR source/This control specifies the string parameter containing source path and file
ViStatus _VI_FUNC rsrtm20xx_DisplayAddLanguage (ViSession instrSession,
                                            ViInt32 language, ViString source)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      cmd [RSRTM20XX_IO_BUFFER_SIZE] = "";
    
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, language, 
        RSRTM20XX_VAL_DISPLAY_ENGLISH, RSRTM20XX_VAL_DISPLAY_KOREAN), 2, "Language");
  
	sprintf (cmd, "DISP:LANG:ADD %s,\"%s\"", languageArr[language], source);
	checkErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
	checkErr (rsrtm20xx_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function configures the markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR markerAction/This Control selects the marker action.
ViStatus _VI_FUNC rsrtm20xx_MarkerSearch (ViSession instrSession,
                                              ViInt32 markerAction)
{
    ViStatus    error = VI_SUCCESS;

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, markerAction, 
        RSRTM20XX_VAL_MARKER_ACTION_SET, RSRTM20XX_VAL_MARKER_ACTION_ACLEAR), 2, "Marker Action");
 
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch (markerAction)
    {
        case RSRTM20XX_VAL_MARKER_ACTION_SET:
            viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                RSRTM20XX_ATTR_MARKER_SET, VI_NULL));
            break;
        case RSRTM20XX_VAL_MARKER_ACTION_NEXT:
            viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                RSRTM20XX_ATTR_MARKER_NEXT, VI_NULL));
            break;
        case RSRTM20XX_VAL_MARKER_ACTION_PREV:
            viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                RSRTM20XX_ATTR_MARKER_PREVIOUS, VI_NULL));
            break;
        case RSRTM20XX_VAL_MARKER_ACTION_CLEAR:
            viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                RSRTM20XX_ATTR_MARKER_CLEAR, VI_NULL));
            break;
        case RSRTM20XX_VAL_MARKER_ACTION_ACLEAR:
            viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                RSRTM20XX_ATTR_MARKER_CLEAR_ALL, VI_NULL));
            break;

        default:
            break;
    }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the absolute daytime of the selected acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR hour/Returns hour.
/// HIPAR minute/Returns minute.
/// HIPAR second/Returns seconds.
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryChannelAcquisitionAbsoluteTime (ViSession instrSession,
                                                                        ViInt32 channel,
                                                                        ViInt32 *hour,
                                                                        ViInt32 *minute,
                                                                        ViReal64 *second)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, channel, RSRTM20XX_VAL_CHANNEL_1, RSRTM20XX_VAL_CHANNEL_EXT), 2, "Channel");

	sprintf (command, "CHAN%ld:HIST:TSAB?\n", channel);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (hour != NULL) *hour = atol (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (minute != NULL) *minute = atol (p2buf);

	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (second != NULL) *second = atof (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the date of the selected acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR year/Returns year.
/// HIPAR month/Returns month.
/// HIPAR day/Returns day.
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryChannelAcquisitionDate (ViSession instrSession,
                                                                ViInt32 channel,
                                                                ViInt32 *year,
                                                                ViInt32 *month,
                                                                ViInt32 *day)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, channel, RSRTM20XX_VAL_CHANNEL_1, RSRTM20XX_VAL_CHANNEL_EXT), 2, "Channel");
   
	sprintf (command, "CHAN%ld:HIST:TSD?\n", channel);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (year != NULL) *year = atol (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (month != NULL) *month = atol (p2buf);

	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (day != NULL) *day = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the absolute daytime of the selected acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR digitalChannel/Selects digital channel.
/// HIPAR hour/Returns hour.
/// HIPAR minute/Returns minute.
/// HIPAR second/Returns seconds.
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryDigitalAcquisitionAbsoluteTime (ViSession instrSession,
                                                                        ViInt32 digitalChannel,
                                                                        ViInt32 *hour,
                                                                        ViInt32 *minute,
                                                                        ViReal64 *second)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, digitalChannel, 0, 15), 2, "Digital Channel");
    
	sprintf (command, "DIG%ld:HIST:TSAB?\n", digitalChannel);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (hour != NULL) *hour = atol (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (minute != NULL) *minute = atol (p2buf);

	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (second != NULL) *second = atof (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;

}

/// HIFN  This function returns the date of the selected acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR digitalChannel/Selects digital channel.
/// HIPAR year/Returns year.
/// HIPAR month/Returns month.
/// HIPAR day/Returns day.
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryDigitalAcquisitionDate (ViSession instrSession,
                                                                ViInt32 digitalChannel,
                                                                ViInt32 *year,
                                                                ViInt32 *month,
                                                                ViInt32 *day)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, digitalChannel, 0, 15), 2, "Digital Channel");

	sprintf (command, "DIG%ld:HIST:TSD?\n", digitalChannel);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (year != NULL) *year = atol (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (month != NULL) *month = atol (p2buf);

	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (day != NULL) *day = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the absolute daytime of the selected acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR hour/Returns hour.
/// HIPAR minute/Returns minute.
/// HIPAR second/Returns seconds.
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryMathAcquisitionAbsoluteTime (ViSession instrSession,
                                                                     ViInt32 mathNumber,
                                                                     ViInt32 *hour,
                                                                     ViInt32 *minute,
                                                                     ViReal64 *second)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, mathNumber, 1, 4), 2, "Math Number");
    
	sprintf (command, "CALC:MATH%ld:HIST:TSAB?\n", mathNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (hour != NULL) *hour = atol (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (minute != NULL) *minute = atol (p2buf);

	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (second != NULL) *second = atof (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the date of the selected acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR year/Returns year.
/// HIPAR month/Returns month.
/// HIPAR day/Returns day.
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryMathAcquisitionDate (ViSession instrSession,
                                                             ViInt32 mathNumber,
                                                             ViInt32 *year,
                                                             ViInt32 *month,
                                                             ViInt32 *day)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, mathNumber, 1, 4), 2, "Math Number");

	sprintf (command, "CALC:MATH%ld:HIST:TSD?\n", mathNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (year != NULL) *year = atol (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (month != NULL) *month = atol (p2buf);

	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (day != NULL) *day = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the absolute daytime of the selected acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR hour/Returns hour.
/// HIPAR minute/Returns minute.
/// HIPAR second/Returns seconds.
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryProtocolAcquisitionAbsoluteTime (ViSession instrSession,
                                                                         ViInt32 serialBus,
                                                                         ViInt32 *hour,
                                                                         ViInt32 *minute,
                                                                         ViReal64 *second)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, serialBus, RSRTM20XX_VAL_SERIAL_BUS_1, RSRTM20XX_VAL_SERIAL_BUS_4), 2, "Serial Bus");
    
	sprintf (command, "BUS%ld:HIST:TSAB?\n", serialBus);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (hour != NULL) *hour = atol (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (minute != NULL) *minute = atol (p2buf);

	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (second != NULL) *second = atof (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the date of the selected acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR year/Returns year.
/// HIPAR month/Returns month.
/// HIPAR day/Returns day.
ViStatus _VI_FUNC rsrtm20xx_QueryHistoryProtocolAcquisitionDate (ViSession instrSession,
                                                                 ViInt32 serialBus,
                                                                 ViInt32 *year,
                                                                 ViInt32 *month,
                                                                 ViInt32 *day)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, serialBus, RSRTM20XX_VAL_SERIAL_BUS_1, RSRTM20XX_VAL_SERIAL_BUS_4), 2, "Serial Bus");
	
	sprintf (command, "BUS%ld:HIST:TSD?\n", serialBus);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (year != NULL) *year = atol (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (month != NULL) *month = atol (p2buf);

	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (day != NULL) *day = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the state of the indicated measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR signalSource/Selects the source of the measurement.
/// HIPAR referenceSource/Selects the source of the reference.
ViStatus _VI_FUNC rsrtm20xx_ConfigureMeasurementSource(
    ViSession   instrSession,
    ViInt32     measurement,
    ViInt32     signalSource,
    ViInt32     referenceSource
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, measurement, RSRTM20XX_VAL_MEASUREMENT_1, 
        RSRTM20XX_VAL_MEASUREMENT_4), 2, "Measurement");
    
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, signalSource, RSRTM20XX_VAL_MEASUREMENT_SOURCE_NONE, 
        RSRTM20XX_VAL_MEASUREMENT_SOURCE_AVER), 3, "Signal Source");
   
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, referenceSource, RSRTM20XX_VAL_MEASUREMENT_SOURCE_NONE, 
        RSRTM20XX_VAL_MEASUREMENT_SOURCE_AVER), 4, "Reference Source");
    
    viCheckErr (viPrintf(instrSession, "MEAS%ld:SOUR %s,%s", 
        measurement, signalSourceArr[signalSource], signalSourceArr[referenceSource]));
    
    viCheckErr(rsrtm20xx_CheckStatusTimed(instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR quickMeasurementsEnabled/This function enables and disables all quick measurements of the indicated measurement
ViStatus _VI_FUNC rsrtm20xx_ConfigureQuickMeasurements (ViSession instrSession,
                                                    ViInt32 measurement,
                                                    ViBoolean quickMeasurementsEnabled)
{
    ViStatus    error = VI_SUCCESS;
	ViChar   rep_cap_measurement[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_MEASUREMENT_ENABLED,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");
	
    if (quickMeasurementsEnabled)
    {
        viCheckParm(rsrtm20xx_SetAttributeViString(instrSession, rep_cap_measurement,
        RSRTM20XX_ATTR_QUICK_MEASUREMENTS_ENABLE, VI_NULL), 3, "Quick Measurements Enable");
    }
    else
    {
        viCheckParm(rsrtm20xx_SetAttributeViString(instrSession, rep_cap_measurement,
        RSRTM20XX_ATTR_QUICK_MEASUREMENTS_DISABLE, VI_NULL), 3, "Quick Measurements Disable");
    }
Error:
	Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
} 

/// HIFN  This function returns the results of the quick measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR results/Returns the list of values.
ViStatus _VI_FUNC rsrtm20xx_QueryQuickMeasurementResults (ViSession instrSession,
                                            ViInt32 measurement,
                                            ViInt32 arraySize,
                                            ViReal64 results[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *buffer = VI_NULL;
    ViChar      *pbuff = VI_NULL;
    ViInt32     i = 0;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, measurement, RSRTM20XX_VAL_MEASUREMENT_1, 
        RSRTM20XX_VAL_MEASUREMENT_4), 2, "Measurement");
    
    checkErr (viPrintf (instrSession, "MEAS%ld:ARES?\n", measurement));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &buffer, VI_NULL)); 
    
    pbuff = strtok (buffer, ",");
    
    while (pbuff != NULL && (i<arraySize))
    {
        if (i<arraySize)
        {
            results[i++] = atof (pbuff);
            pbuff = strtok (NULL, ",");
        }
        i++;
    } 
	
    viCheckErr(rsrtm20xx_CheckStatus(instrSession));
Error:
    if (buffer) free (buffer);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the statistical results for the selected measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function. 
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
ViStatus _VI_FUNC rsrtm20xx_ResetAutomaticMeasurement (ViSession instrSession,
                                                   ViInt32 measurement)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap_measurement[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_RESET_AUTOMATIC_MEASUREMENT,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap_measurement), 2, "Measurement");

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, rep_cap_measurement,
        									RSRTM20XX_ATTR_RESET_AUTOMATIC_MEASUREMENT, VI_NULL));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the edges to be used for delay measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR source1Slope/Slope of source 1 (first waveform).
/// HIPAR source2Slope/Slope of source 2 (second waveform).
ViStatus _VI_FUNC rsrtm20xx_ConfigureDelayMeasuremenSlope (ViSession instrSession,
                                                       ViInt32 measurement,
                                                       ViInt32 source1Slope,
                                                       ViInt32 source2Slope)
{
	ViStatus    error = VI_SUCCESS;
    ViString	slopeArr[] = {"POS","NEG"};
	
	checkErr (Rs_LockSession (instrSession, VI_NULL)); 
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, measurement, 1, 8), 2, "Measurement");
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, source1Slope, RSRTM20XX_VAL_MEAS_SLOPE_POS, 
											RSRTM20XX_VAL_MEAS_SLOPE_NEG), 3, "Source 1 Slope");
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, source2Slope, RSRTM20XX_VAL_MEAS_SLOPE_POS, 
											RSRTM20XX_VAL_MEAS_SLOPE_NEG), 4, "Source 2 Slope");
    
	viCheckErr (viPrintf(instrSession, "MEAS%ld:DEL:SLOP %s,%s\n", 
						measurement, slopeArr[source1Slope], slopeArr[source2Slope]));

    checkErr (rsrtm20xx_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the lower and upper reference levels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR relativeMode/Selects the reference level.
ViStatus _VI_FUNC rsrtm20xx_ConfigureReferenceLevelMode (ViSession instrSession,
                                                     ViInt32 relativeMode)
{
	ViStatus    error = VI_SUCCESS;

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_MEAS_RELATIVE_MODE, relativeMode), 3, "Relative Mode");  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Set the lower and upper reference levels for rise and fall time mesurements,
/// HIFN  and the middle reference level used for phase and delay measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR lowerLevel/Set the lower reference level for rise and fall time mesurements.
/// HIPAR upperLevel/Set the upper reference level for rise and fall time mesurements.
/// HIPAR middleLevel/Set the middle reference level used for phase and delay measurements.
ViStatus _VI_FUNC rsrtm20xx_ConfigureUserReferenceLevel (ViSession instrSession,
                                                     ViReal64 lowerLevel,
                                                     ViReal64 upperLevel,
                                                     ViReal64 middleLevel)
{
	ViStatus    error = VI_SUCCESS;

    viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_MEAS_RELATIVE_LOWER_LEVEL, lowerLevel), 2, "Lower Level");  
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_MEAS_RELATIVE_UPPER_LEVEL, upperLevel), 3, "Upper Level");
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_MEAS_RELATIVE_MIDDLE_LEVEL, middleLevel), 4, "Middle Level");    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function switches all cursors off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session.
ViStatus _VI_FUNC rsrtm20xx_AllCursorsOff(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    viCheckParm(rsrtm20xx_SetAttributeViString(instrSession, "",
        RSRTM20XX_ATTR_ALL_CURSORS_OFF, VI_NULL), 2, "All Cursors Off");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the cursor to the initial position or sets the cursor lines to typical points of the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function. 
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR position/Selects if the cursor position resets or autosets.
ViStatus _VI_FUNC rsrtm20xx_SetCursorPosition (ViSession instrSession,
                                           ViInt32 cursor, ViInt32 position)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, position, 0, 1), 3, "Position");
   
	viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_CURSOR_AUTOSET,
                                                 "Cursor",
                                                 cursor,
                                                 15,
                                                 rep_cap), 2, "Cursor");

	switch(position) 
    {
        case RSRTM20XX_VAL_CURSOR_POSITION_AUTOSET:
            viCheckErr(rsrtm20xx_SetAttributeViString (instrSession, "", RSRTM20XX_ATTR_CURSOR_AUTOSET, VI_NULL));   
            
        break;
            
        case RSRTM20XX_VAL_CURSOR_POSITION_RESET:
            viCheckErr(rsrtm20xx_SetAttributeViString(instrSession, "", RSRTM20XX_ATTR_CURSOR_RESET, VI_NULL));   
        break;
            
        default: 
        break;
    }
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function configures the horizontal and vertical position of
/// HIFN selected cursor or cursor line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function. 
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR xPosition1/Defines the position of the left vertical cursor line.
/// HIPAR xPosition2/Defines the position of the right vertical cursor line. 
/// HIPAR yPosition1/Defines the position of the lower horizontal cursor line.
/// HIPAR yPosition2/Defines the position of the upper horizontal cursor line. 
ViStatus _VI_FUNC rsrtm20xx_ConfigureCursorPosition (ViSession instrSession,
                                                     ViReal64 xPosition1,
                                                     ViReal64 xPosition2,
                                                     ViReal64 yPosition1,
                                                     ViReal64 yPosition2)
{
	ViStatus    error = VI_SUCCESS;

    viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_CURSOR_X1_POSITION, xPosition1), 2, "xPosition1");  
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_CURSOR_X2_POSITION, xPosition2), 3, "xPosition2");
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_CURSOR_Y1_POSITION, yPosition1), 4, "yPosition1");
	
	viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_CURSOR_Y2_POSITION, yPosition2), 5, "yPosition2");  
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function sets the cursor to the next or the previous peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR instrSession/The handle identifies a particular instrument session.
/// HIPAR cursor/This control selects the cursor.
/// HIPAR peak/This control selects if the cursor is set to the next or the previous peak.
ViStatus _VI_FUNC rsrtm20xx_SetFFTAnalysisCursorToPeak (ViSession instrSession,
                                                    ViInt32 cursor, ViInt32 peak)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, peak, 0, 1), 3, "Position");
    
	viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_CURSOR_AUTOSET,
                                                 "Cursor",
                                                 cursor,
                                                 15,
                                                 rep_cap), 2, "Cursor");

	switch(peak) 
    {
        case RSRTM20XX_VAL_CURSOR_PEAK_NEXT:
            viCheckErr (rsrtm20xx_SetAttributeViString(instrSession, "", RSRTM20XX_ATTR_CURSOR_PEAK_NEXT, VI_NULL));   
            
        break;
        case RSRTM20XX_VAL_CURSOR_PEAK_PREVIOUS:
            viCheckErr (rsrtm20xx_SetAttributeViString(instrSession, "", RSRTM20XX_ATTR_CURSOR_PEAK_PREVIOUS, VI_NULL));   
        break;
        default: 
        break;
    }
   
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Defines the main measurement. This measurement is the one referred to 
/// HIFN  if the measurement waveform is used as a source for math calculations. 
/// HIFN  The main measurement type must belong to the same category as the other 
/// HIFN  types assigned to the same measurement waveform, if there are any.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Defines or queries the main measurement. This measurement is the one 
/// HIPAR measurementFunction/referred to if the measurement waveform is used as a source for math 
/// HIPAR measurementFunction/calculations. The main measurement type must belong to the same 
/// HIPAR measurementFunction/category as the other types assigned to the same measurement waveform, 
/// HIPAR measurementFunction/if there are any.
ViStatus _VI_FUNC rsrtm20xx_ConfigureAmplitudeTimeMainMeasurementRTM(
    ViSession   instrSession,
    ViInt32 measurement,
    ViInt32 measurementFunction)
{
    ViStatus error = VI_SUCCESS;
    ViChar      model [RSRTM20XX_IO_BUFFER_SIZE] = ""; 
    ViChar   rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_GetAttributeViString (instrSession, "",
        RS_ATTR_INSTRUMENT_MODEL, RSRTM20XX_IO_BUFFER_SIZE, model));

    if (strstr (model, "RTO"))
        viCheckErr (RS_ERROR_INSTRUMENT_MODEL);
    
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, measurement, RSRTM20XX_VAL_MEASUREMENT_1, 
                                                  RSRTM20XX_VAL_MEASUREMENT_8), 2, "Measurement");
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_AMPT_MAIN_MEASUREMENT,
                                                 "Measurement",
                                                 measurement - 1,
                                                 15,
                                                 rep_cap), 2, "Measurement");

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_AMPT_MAIN_MEASUREMENT, measurementFunction), 3, "Amplitude Time Measurement Function");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Creates a mask from the envelope waveform of the test source set with MASK: SOURce.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_MaskCopyChannel (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, NULL,
        RSRTM20XX_ATTR_MASK_COPY_CHANNEL, VI_NULL ));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Sets the counters of passed and failed acquisitions to Zero.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_MaskResetTestCounter (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
        RSRTM20XX_ATTR_MASK_RESET_TEST_COUNTER, NULL));

Error:
	Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the result values of the specified search result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR resultNumber/Number of result to be read. 
/// HIPAR xPosition/Returns the X-position (time) of the search result
/// HIPAR yPosition/Returns the Y-position of the search result, currently not relevant. .
/// HIPAR resultType/Returns the type of the search result (Edge, Peak, Width).
/// HIPAR polarity/Returns the slope or polarity of the search result.
/// HIPAR voltageWidth/For peak searches, the value contains the peak voltage.
ViStatus _VI_FUNC rsrtm20xx_QuerySearchResult (ViSession instrSession,
                                           ViInt32 resultNumber,
                                           ViReal64 *xPosition,
                                           ViReal64 *yPosition,
                                           ViInt32 *resultType,
                                           ViInt32 *polarity,
                                           ViReal64 *voltageWidth)
{
	ViStatus error = VI_SUCCESS;
    ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	ViString resultTypeArr[] = {"EDGE","WIDTH","PEAK",VI_NULL};
	ViString polarityArr[] = {"POSITIVE","NEGATIVE","EITHER",VI_NULL}; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    sprintf (command, "SEAR:RES%ld?\n", resultNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xPosition != NULL) *xPosition = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (yPosition != NULL) *yPosition = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (resultType != NULL) *resultType = rsrtm20xx_StringIndex (resultTypeArr, p2buf);;

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (polarity != NULL) *polarity = rsrtm20xx_StringIndex (polarityArr, p2buf);;
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (voltageWidth != NULL) *voltageWidth = atof (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}



/// HIFN  This function returns all result values of the search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR arraySize/Pass the number of elements in the array parameters.
/// HIPAR xPosition/Returns the X-position (time) of the search result
/// HIPAR yPosition/Returns the Y-position of the search result, currently not relevant. .
/// HIPAR resultType/Returns the type of the search result (Edge, Peak, Width).
/// HIPAR polarity/Returns the slope or polarity of the search result.
/// HIPAR voltageWidth/For peak searches, the value contains the peak voltage.
/// HIPAR numberOfResults/Number of returned results.
ViStatus _VI_FUNC rsrtm20xx_QueryAllSearchResults (ViSession instrSession,
                                               ViInt32 arraySize,
                                               ViReal64 xPosition[],
                                               ViReal64 yPosition[],
                                               ViInt32 resultType[],
                                               ViInt32 polarity[],
                                               ViReal64 voltageWidth[],
                                               ViInt32 *numberOfResults)
{
	ViStatus error = VI_SUCCESS;
    ViChar   command[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
	ViInt32	 i = 0;
	ViString resultTypeArr[] = {"EDGE","WIDTH","PEAK",VI_NULL};
	ViString polarityArr[] = {"POSITIVE","NEGATIVE","EITHER",VI_NULL}; 

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    sprintf (command, "SEAR:RES:ALL?\n");
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
    
	p2buf = strtok (p2data, ",");
	
    for (i=0; i<arraySize; i++)
	{
		p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    xPosition[i] = atof (p2buf);
    
	    p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    yPosition[i] = atof (p2buf);

	    p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    resultType[i] = rsrtm20xx_StringIndex (resultTypeArr, p2buf);;

	    p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    polarity[i] = rsrtm20xx_StringIndex (polarityArr, p2buf);;
	
		p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        checkErr (VI_ERROR_INV_RESPONSE);
	    voltageWidth[i] = atof (p2buf);
		*numberOfResults = i;
		p2buf = strtok (NULL, ",");
		if (p2buf == NULL) break;
	}

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Executes the analysis of all channels that are configured for the selected bus 
/// HIFN and sets the threshold for digitization of analog signals for each channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus _VI_FUNC rsrtm20xx_ProtocolFindLevel (ViSession instrSession,
                                           ViInt32 channel)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_PROTOCOL_FIND_LEVEL,
                                                 "Channel", channel,
                                                 15, rep_cap), 2, "Channel");

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, rep_cap,
        							RSRTM20XX_ATTR_PROTOCOL_FIND_LEVEL, VI_NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function queries the contents of the frame table in block data format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control indicates the allocated memory in Decoded Data.
/// HIPAR decodedData/Returns the contents of the frame table in block data format.
ViStatus _VI_FUNC rsrtm20xx_QueryBusDecodedData (ViSession instrSession,
                                        ViInt32 serialBus, ViInt32 bufferSize,
                                        ViChar decodedData[])
{
	ViStatus    error = VI_SUCCESS;
	ViChar      buffer [RS_MAX_MESSAGE_BUF_SIZE] = "";
	ViUInt32    len = 0;
	ViChar*     extBuf = NULL;
	ViUInt32    ret_cnt = 0;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	sprintf (buffer, "BUS%ld:LIST?\n", serialBus);
	
	checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt)); 
    if (buffer[0]=='#')
    {
        /* Disable termination character */
        checkErr (viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
        /* Parse header */
        checkErr (viRead (instrSession, (ViBuf) buffer, 1, &ret_cnt));
        buffer[1] = '\0';
        sscanf (buffer, "%ld", &len);
        checkErr (viRead (instrSession, (ViBuf) buffer, len, &ret_cnt));
        buffer[len] = '\0';
        sscanf (buffer, "%ld", &len);
        
        if (len>0)
        {
            checkAlloc (extBuf = (ViChar*) malloc (len));
            /* Read useful data */
            checkErr (viRead (instrSession, (ViBuf) extBuf, len, &ret_cnt));
    
            if (len != ret_cnt)
                viCheckErr (RS_ERROR_UNEXPECTED_RESPONSE);

            /* Flush garbage */
            viScanf (instrSession,"%*[^\r\n]");
		}
	}
	else
		checkErr(VI_ERROR_INV_RESPONSE);
	
	len = (len <= (ViUInt32) bufferSize) ? len : (ViUInt32) bufferSize;
	strncpy (decodedData, extBuf, (size_t) len);
	
	error = rsrtm20xx_CheckStatus (instrSession);

Error:
	if (extBuf) free (extBuf);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the all data words of the specified frame in comma-separated list of decimal values of the data bytes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Data words of the specified frame.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsrtm20xx_QueryI2CFrameAllData (ViSession instrSession,
                                         ViInt32 serialBus, ViInt32 frame,
                                         ViInt32 arraySize, ViInt32 data[],
                                         ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      *p2buf  = NULL;
    ViChar      *extBuf=VI_NULL; 
    ViUInt32    ret_cnt;
    ViInt32     i=0;
    ViInt32     value_count = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
    
    viCheckErr( viPrintf(instrSession, "BUS%ld:I2C:FRAM%d:DATA?", serialBus, frame));
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &extBuf, &ret_cnt)); 
    
    p2buf = strtok(extBuf, ",");
    
    value_count = atoi (p2buf);
    
    arraySize = (arraySize < value_count) ? arraySize : value_count;
    
    p2buf = strtok(NULL, ",");
    
    while (p2buf && (i < arraySize))
    {                  
        data[i] = atol(p2buf);
        p2buf = strtok(NULL, ",");
        i++;
    }
    *returnedValues = i;
	
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data words of the specified frame of the MISO or MOSI line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or rsrtm20xx_InitWithOptions function.
/// HIPAR line/This control selects line.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Data words of the specified frame.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsrtm20xx_QuerySPIFrameAllData (ViSession instrSession,
                                         ViInt32 serialBus, ViInt32 line,
                                         ViInt32 frame, ViInt32 arraySize,
                                         ViInt32 data[], ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      *p2buf  = NULL; 
    ViChar      *extBuf=VI_NULL; 
    ViUInt32    ret_cnt;
    ViInt32     i=0;
    ViInt32     value_count = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K1")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, line, RSRTM20XX_VAL_SPI_MISO, RSRTM20XX_VAL_SPI_MOSI), 2, "Line");
    
    if (line == RSRTM20XX_VAL_SPI_MISO)
	{
		viCheckErr( viPrintf(instrSession, "BUS%ld:SPI:FRAM%d:DATA:MISO?", serialBus, frame));
	}
	else
	{
		viCheckErr( viPrintf(instrSession, "BUS%ld:SPI:FRAM%d:DATA:MOSI?", serialBus, frame));
	}
	
    viCheckErr (Rs_ReadDataUnknownLength(instrSession, &extBuf, &ret_cnt)); 
    
    p2buf = strtok(extBuf, ",");
    
    value_count = atoi (p2buf);
    
    arraySize = (arraySize < value_count) ? arraySize : value_count;
    
    p2buf = strtok(NULL, ",");
    
    while (p2buf && (i < arraySize))
    {                  
        data[i] = atol(p2buf);
        p2buf = strtok(NULL, ",");
        i++;
    }
    *returnedValues = i;
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsrtm20xx_QueryLINFrameAllData (ViSession instrSession,
                                         ViInt32 serialBus, ViInt32 frame,
                                         ViInt32 arraySize, ViInt32 data[],
                                         ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      *p2buf  = NULL; 
    ViChar      *extBuf=VI_NULL; 
    ViUInt32    ret_cnt; 
    ViInt32     i=0;
    ViInt32     value_count = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr( viPrintf(instrSession, "BUS%ld:LIN:FRAM%d:DATA?", serialBus, frame));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &extBuf, &ret_cnt)); 
    
    p2buf = strtok (extBuf, ",");
    
    value_count = atoi (p2buf);
    
    arraySize = (arraySize < value_count) ? arraySize : value_count;
    
    p2buf = strtok(NULL, ",");
    
    while (p2buf && (i < arraySize))
    {                  
        data[i] = atol(p2buf);
        p2buf = strtok(NULL, ",");
        i++;
    }
    *returnedValues = i;
	 
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Queries the data bytes of the specified frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR frame/This control selects the frame.
/// HIPAR arraySize/Pass the number of values allocated for Data result array.
/// HIPAR data/Array of decimal values of the data bytes.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rsrtm20xx_QueryCANFrameAllData (ViSession instrSession,
                                         ViInt32 serialBus, ViInt32 frame,
                                         ViInt32 arraySize, ViInt32 data[],
                                         ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      buffer [RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      *p2buf  = NULL; 
    ViChar      *extBuf=VI_NULL; 
    ViUInt32    ret_cnt; 
    ViInt32     i=0;
    ViInt32     value_count = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K3")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckErr (viPrintf (instrSession, "BUS%ld:CAN:FRAM%ld:DATA?", serialBus, frame));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &extBuf, &ret_cnt)); 
    
    p2buf = strtok(extBuf, ",");
    
    value_count = atoi (p2buf);
    
    arraySize = (arraySize < value_count) ? arraySize : value_count;
    
    p2buf = strtok(NULL, ",");
    
    while (p2buf && (i < arraySize))
    {                  
        data[i] = atol (p2buf);
        p2buf = strtok (NULL, ",");
        i++;
    }
    *returnedValues = i;
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets all selected tracks to the middle of the display and scales them 
/// HIFN  to full height of the display (8 dovisions). The track waveforms 
/// HIFN  overlap.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsrtm20xx_ProtocolAudioTrackWaveformSetToDefault(
	ViSession	instrSession,
	ViInt32	serialBus
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                     		RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_SET_TO_DEFAULT,
                                                 "SerialBus", serialBus - 1,
                                                 15, rep_cap), 2, "SerialBus");

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, rep_cap,
        		RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_SET_TO_DEFAULT, VI_NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  Arranges the selected tracks vertically, one above the other. The 
/// HIFN  track waveforms do not overlap.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
ViStatus _VI_FUNC rsrtm20xx_ProtocolAudioTrackWaveformSetToScreen(
	ViSession	instrSession,
	ViInt32	serialBus
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                     		RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_SET_TO_SCREEN,
                                                 "SerialBus", serialBus - 1,
                                                 15, rep_cap), 2, "SerialBus");

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, rep_cap,
        		RSRTM20XX_ATTR_PROTOCOL_AUDIO_TRACK_WAVEFORM_SET_TO_SCREEN, VI_NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
}

/// HIFN  Sets the channel for the signal source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR source/Sets the channel for the signal source.
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429Source (ViSession instrSession,
                                                     ViInt32 serialBus,
                                                     ViInt32 source)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                     		RSRTM20XX_ATTR_PROTOCOL_ARINC429_SOURCE,
                                                 "SerialBus", serialBus - 1,
                                                 15, rep_cap), 2, "SerialBus");

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_PROTOCOL_ARINC429_SOURCE, source), 3, "Source");  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     

}

/// HIFN  This function selects the wire on which the bus signal is measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR polarity/Sets the wire on which the bus signal is measured.
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429Polarity (ViSession instrSession,
                                                       ViInt32 serialBus,
                                                       ViInt32 polarity)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                     		RSRTM20XX_ATTR_PROTOCOL_ARINC429_POLARITY,
                                                 "SerialBus", serialBus - 1,
                                                 15, rep_cap), 2, "SerialBus");

    viCheckParm (rsrtm20xx_SetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_PROTOCOL_ARINC429_POLARITY, polarity), 3, "Polarity");  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error; 
}

/// HIFN  This function configures the bit rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR bitRateMode/Sets the bit rate mode to high or low speed.
/// HIPAR bitRateValue/Sets the number of transmitted bits per second.
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429BitRate (ViSession instrSession,
                                                      ViInt32 serialBus,
                                                      ViInt32 bitRateMode,
                                                      ViReal64 bitRateValue)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   rep_cap[15] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
	                                                 RSRTM20XX_ATTR_PROTOCOL_ARINC429_BIT_RATE_MODE,
	                                                 "SerialBus",
	                                                 serialBus - 1,
	                                                 15,
	                                                 rep_cap), 2, "SerialBus"); 
    
	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, rep_cap,
	        RSRTM20XX_ATTR_PROTOCOL_ARINC429_BIT_RATE_MODE, bitRateMode), 3, "Bit Rate Mode");
	
	if(bitRateMode == RSRTM20XX_VAL_ARINC429_BIT_RATE_MODE_USER)
	{
		viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, rep_cap,
        RSRTM20XX_ATTR_PROTOCOL_ARINC429_BIT_RATE, bitRateValue), 4, "Bit Rate Value");
	}
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures threshold values used for digitization of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/This control selects the serial bus.
/// HIPAR thresholdHigh/Sets the high threshold level of the signal.
/// HIPAR thresholdLow/Sets the lower threshold level of the signal.
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429Threshold (ViSession instrSession,
                                                        ViInt32 serialBus,
                                                        ViReal64 thresholdHigh,
                                                        ViReal64 thresholdLow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   rep_cap[15] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
	                                                 RSRTM20XX_ATTR_PROTOCOL_ARINC429_THRESHOLD_HIGH,
	                                                 "SerialBus",
	                                                 serialBus - 1,
	                                                 15,
	                                                 rep_cap), 2, "SerialBus"); 
    
	viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
	        RSRTM20XX_ATTR_PROTOCOL_ARINC429_THRESHOLD_HIGH, thresholdHigh), 3, "Threshold High");
	
	viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, rep_cap,
	        RSRTM20XX_ATTR_PROTOCOL_ARINC429_THRESHOLD_LOW, thresholdLow), 4, "Threshold Low");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the trigger type for ARINC 429 analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the type of trigger.
ViStatus _VI_FUNC rsrtm20xx_ConfigureARINC429TriggerType (ViSession instrSession,
                                                          ViInt32 type)
{
	ViStatus	error = VI_SUCCESS;

    viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, NULL,
        RSRTM20XX_ATTR_PROTOCOL_ARINC429_TRIGGER_TYPE, type), 2, "Type");  

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;

}

/// HIFN  Returns the number of decoded words.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects the serial bus.
/// HIPAR count/Returns the number of decoded words.
ViStatus _VI_FUNC rsrtm20xx_QueryARINC429WordCount (ViSession instrSession,
                                                    ViInt32 serialBus,
                                                    ViInt32 *count)
{
	ViStatus	error = VI_SUCCESS;
	ViChar   rep_cap[15] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
	                                                 RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_COUNT,
	                                                 "SerialBus",
	                                                  serialBus - 1,
	                                                 15,
	                                                 rep_cap), 2, "SerialBus"); 
    
	viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, rep_cap,
	        RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_COUNT, count), 3, "Count");
	
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Queries the overall state of the word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR status/Returns the status of the specified word.
/// HIPAR start/Returns the start time of the specified word.
/// HIPAR stop/Returns the start time of the specified word.
/// HIPAR pattern/Returns all 32 bits of a data word as decimal value.
/// HIPAR SDI/Returns the source/destination identifier (SDI) bits of the specified word.
/// HIPAR SSM/Returns the source/destination identifier (SSM) bits of the specified word.
ViStatus _VI_FUNC rsrtm20xx_QueryARINC429Status (ViSession instrSession,
                                                 ViInt32 serialBus, ViInt32 word,
                                                 ViInt32 *status,
                                                 ViReal64 *start, ViReal64 *stop,
                                                 ViInt32 *pattern, ViInt32 *SDI,
                                                 ViInt32 *SSM)
{
	ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                     		RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_STATUS,
                                                 "SerialBus", serialBus - 1,
                                                   5, rep_cap), 2, "SerialBus");

	viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, "",
	        RSRTM20XX_ATTR_PROTOCOL_ARINC429_SELECTED_WORD, word), 3, "Word");
	
    viCheckParm (rsrtm20xx_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_STATUS, status
                                               ), 4, "Status");
	
	viCheckParm (rsrtm20xx_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_START, start
                                               ), 5, "Start");
	
	viCheckParm (rsrtm20xx_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_STOP, stop
                                               ), 6, "Stop");
	
	viCheckParm (rsrtm20xx_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_PATTERN, pattern
                                               ), 7, "Pattern");
	
	viCheckParm (rsrtm20xx_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_SDI, SDI
                                               ), 8, "SDI");
	
	viCheckParm (rsrtm20xx_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSRTM20XX_ATTR_PROTOCOL_ARINC429_WORD_SSM, SSM
                                               ), 9, "SSM");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Return the data bytes of the specified word.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or 
/// HIPAR instrSession/rsscope_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR serialBus/Selects the serial bus.
/// HIPAR word/This control selects the word.
/// HIPAR arraySize/Returns the status of the specified word.
/// HIPAR data/Returns the start time of the specified word.
/// HIPAR returnedValues/Returns the start time of the specified word.
ViStatus _VI_FUNC rsrtm20xx_QueryARINC429AllData (ViSession instrSession,
                                                  ViInt32 serialBus,
                                                  ViInt32 word,
                                                  ViInt32 arraySize,
                                                  ViInt32 data[],
                                                  ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      *p2buf  = NULL; 
    ViChar      *extBuf=VI_NULL; 
    ViUInt32    ret_cnt; 
    ViInt32     i=0;
    ViInt32     value_count = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        RSRTM20XX_ATTR_PROTOCOL_ARINC429_SELECTED_WORD, word), 3, "Word");
	
	viCheckErr (viPrintf(instrSession, "BUS%ld:ARIN:WORD%ld:DATA?", serialBus, word));
    viCheckErr (Rs_ReadDataUnknownLength (instrSession, &extBuf, &ret_cnt)); 
    
    p2buf = strtok (extBuf, ",");
    
    value_count = atoi (p2buf);
    
    arraySize = (arraySize < value_count) ? arraySize : value_count;
    
    p2buf = strtok(NULL, ",");
    
    while (p2buf && (i < arraySize))
    {                  
        data[i] = atol(p2buf);
        p2buf = strtok(NULL, ",");
        i++;
    }
    *returnedValues = i;
	 
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function prints an image of the display to the printer or saves 
/// HIFN  an image to a file or the clipboard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Defines the file name when an image of the display is stored to
/// HIPAR fileName/a file rather than printed to a printer using the
/// HIPAR fileName/HCOPy:IMMediate command.
/// HIPAR deviceDestination/This control selects the printer output medium (Disk, Printer or
/// HIPAR deviceDestination/Clipboard).
/// HIPAR includeMenuInScreenshot/Includes the menu in the screenshot. If OFF (no menu), the menu
/// HIPAR includeMenuInScreenshot/is clipped off, and date and time are shown instead of the menu
/// HIPAR includeMenuInScreenshot/name.
/// HIPAR deviceLanguageOutputFormat/Defines the file format for the image of the display.
ViStatus _VI_FUNC rsrtm20xx_ConfigureHardcopySettings (ViSession instrSession,
                                                       ViString fileName,
                                                       ViInt32 deviceDestination,
                                                       ViBoolean includeMenuInScreenshot,
                                                       ViInt32 deviceLanguageOutputFormat)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_SetAttributeViString (instrSession, "",
                                                RSRTM20XX_ATTR_HARDCOPY_FILE_NAME,
                                                fileName), 2, "File Name");

	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
        RSRTM20XX_ATTR_HARDCOPY_DEVICE_DESTINATION, deviceDestination), 3, "Device Destination");
	
	viCheckParm(rsrtm20xx_SetAttributeViBoolean(instrSession, "",
        RSRTM20XX_ATTR_HARDCOPY_INCLUDE_MENU_IN_SCREENSHOT, includeMenuInScreenshot), 4, "Include Menu In Screenshot");
	
	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
        RSRTM20XX_ATTR_HARDCOPY_DEVICE_LANGUAGE_OUTPUT_FORMAT, deviceLanguageOutputFormat), 5, "Device Language Output Format");
	
Error:
	(void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function prints an image of the display to the printer or saves 
/// HIFN  an image to a file or the clipboard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_HardcopyPrint(
    ViSession    instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_SetAttributeViString (instrSession, NULL,
                                                RSRTM20XX_ATTR_HARDCOPY_PRINT,
                                                VI_NULL), 1, "Session");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function defines an instrument name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR name/Defines the name.
ViStatus _VI_FUNC rsrtm20xx_ConfigureInstrumentName (ViSession instrSession,
                                          ViString name)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_SetAttributeViString (instrSession, "",
        RSRTM20XX_ATTR_INSTRUMENT_NAME, name), 2, "Name");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrtm20xx_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufSize/Attribute Value parameter.  
/// HIPAR value/The buffer in which the function returns the current value of the 
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many 
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/// HIFN  This function queries the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViSession variable.
ViStatus _VI_FUNC rsrtm20xx_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt32 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsrtm20xx_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsrtm20xx_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsrtm20xx_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/// HIFN  Performs FFT calculation for the full frequency span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
ViStatus _VI_FUNC rsrtm20xx_ConfigureFFTFrequencySpanFull(
    ViSession   instrSession,
    ViInt32 mathNumber
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, mathNumber, 1, 4), 2, "Math Number");
   
    sprintf (rep_cap, "M%ld", mathNumber);  

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, rep_cap,
                                               RSRTM20XX_ATTR_FFT_FULL_SPAN, NULL));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Copies the selected source waveform with all its settings to the 
/// HIFN  memory of the reference waveform
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceNumber/Selects the reference waveform.
ViStatus _VI_FUNC rsrtm20xx_UpdateReferenceWaveformWithSelectedSource(
    ViSession    instrSession,
    ViInt32    referenceNumber
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_REFERENCE_WAVEFORM_UPDATE,
                                                 "ReferenceWfm",
                                                 referenceNumber - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Number");       

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, rep_cap,
                                               RSRTM20XX_ATTR_REFERENCE_WAVEFORM_UPDATE, NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Loads or saves the reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceWfm/Selects the reference waveform.
/// HIPAR fileName/DDefines the file name.
/// HIPAR operation/This control selects the type of waveform operation to be performed.
ViStatus _VI_FUNC rsrtm20xx_ReferenceWaveformFileOperations (ViSession instrSession,
                                                         ViInt32 referenceWfm,
                                                         ViInt32 operation,
                                                         ViString fileName)
{
    ViStatus    error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_REFERENCE_WAVEFORM_LOAD_FROM_FILE,
                                                 "ReferenceWfm",
                                                 referenceWfm - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Number");       


    switch (operation)
    {
        case RSRTM20XX_VAL_WAV_LOAD_CFG:
            viCheckParm (rsrtm20xx_SetAttributeViString (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_REFERENCE_WAVEFORM_LOAD_CONFIGURATION, NULL
                                                ), 1, "Reference Waveform");
        break;
        
        case RSRTM20XX_VAL_WAV_LOAD:
            viCheckParm (rsrtm20xx_SetAttributeViString (instrSession, rep_cap,
                RSRTM20XX_ATTR_REFERENCE_WAVEFORM_LOAD_FROM_FILE, fileName), 3, "File Name");
        break;
        
        case RSRTM20XX_VAL_WAV_SAVE:
            viCheckParm (rsrtm20xx_SetAttributeViString (instrSession, rep_cap,
                RSRTM20XX_ATTR_REFERENCE_WAVEFORM_SAVE_TO_FILE, fileName), 3, "File Name");
        break;
        default: 
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Operation");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function initiates an acquisition on the channels that you enable 
/// HIFN  with the rsrtm20xx_ConfigureChannel function.  It then waits for the 
/// HIFN  acquisition to complete, and returns the waveform for the channel you 
/// HIFN  specify.  You call the rsrtm20xx_FetchWaveform function to obtain the 
/// HIFN  waveforms for each of the remaining enabled channels without initiating 
/// HIFN  another acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform 
/// HIPAR maximumTime/operation to complete.    
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array.  The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm20xx_ReadWaveform(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 waveformSize,
    ViInt32 maximumTime,
    ViReal64  waveformArray[],
    ViInt32*    actualPoints,
    ViReal64*   initialX,
    ViReal64*   xIncrement
)
{
    ViStatus    error = VI_SUCCESS;
	ViInt32 	oldTimeout = 5;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rsrtm20xx_GetOPCTimeout(instrSession, &oldTimeout));
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, maximumTime));

    checkErr (rsrtm20xx_SetAttributeViString (instrSession, "", RSRTM20XX_ATTR_RUN_SINGLE_WAIT_FOR_OPC, NULL));
	
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, oldTimeout));
	
    checkErr (rsrtm20xx_FetchWaveform (instrSession, channel, waveformSize,
                                     waveformArray, actualPoints, initialX,
                                     xIncrement)); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function initiates measurement and returns datareturns the mask 
/// HIFN  data. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform 
/// HIPAR maximumTime/operation to complete.    
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array.  The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm20xx_ReadMaskData (ViSession instrSession,
                                          ViInt32 waveformSize,
                                          ViInt32 maximumTime,
                                          ViReal64 waveformArray[],
                                          ViInt32 *actualPoints,
                                          ViReal64 *initialX,
                                          ViReal64 *xIncrement)
{
	ViStatus    error = VI_SUCCESS;
	ViInt32 	oldTimeout = 5;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rsrtm20xx_GetOPCTimeout(instrSession, &oldTimeout));
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, maximumTime));

    checkErr (rsrtm20xx_InitiateAcquisition (instrSession, RSRTM20XX_VAL_INIT_SINGLE));
	
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, oldTimeout));
	
    checkErr (rsrtm20xx_FetchMaskData (instrSession, waveformSize,
                                     waveformArray, actualPoints, initialX,
                                     xIncrement)); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;  
}

/// HIFN  This function initiates an acquisition on the channels that you enable 
/// HIFN  with the rsrtm20xx_ConfigureChannel function.  It then waits for the 
/// HIFN  acquisition to complete, obtains a main waveform measurement on the channel 
/// HIFN  you specify, and returns the measurement value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Defines or queries the main measurement.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform 
/// HIPAR maximumTime/operation to complete.    
/// HIPAR measurementResult/Returns the measured value.  
ViStatus _VI_FUNC rsrtm20xx_ReadMainWaveformMeasurement (ViSession instrSession,
                                                          ViInt32 measurement,
                                                          ViInt32 measurementFunction,
                                                          ViInt32 maximumTime,
                                                          ViReal64 *measurementResult)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, measurement, RSRTM20XX_VAL_MEASUREMENT_1, 
                                                  RSRTM20XX_VAL_MEASUREMENT_4), 2, "Measurement");
    
    checkErr (rsrtm20xx_SendSoftwareTrigger (instrSession, maximumTime));

    checkErr (rsrtm20xx_FetchMainWaveformMeasurement (instrSession, 
                                                    measurement,
                                                    measurementFunction,
                                                    measurementResult));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function initiates a waveform acquisition.  After you call this 
/// HIFN  function, the oscilloscope leaves the Idle state and waits for a 
/// HIFN  trigger.  The oscilloscope acquires a waveform for each channel you 
/// HIFN  have enabled with the  rsrtm20xx_ConfigureChannel function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR action/Select action to be performed.
ViStatus _VI_FUNC rsrtm20xx_InitiateAcquisition(
    ViSession   instrSession,
    ViInt32 action
)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    attr = RSRTM20XX_ATTR_RUN_SINGLE;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    switch (action)
    {
        case RSRTM20XX_VAL_INIT_SINGLE:
            attr = RSRTM20XX_ATTR_RUN_SINGLE;
            break;
        case RSRTM20XX_VAL_INIT_CONTINUOUS:
            attr = RSRTM20XX_ATTR_RUN_CONTINUOUS;
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Action");
    }

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "", attr, NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sends software trigger and waits for operation complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR maximumTime/Pass the maximum length of time (in milliseconds) in which to allow 
/// HIPAR maximumTime/the read waveform operation to complete.    
ViStatus _VI_FUNC rsrtm20xx_SendSoftwareTrigger(
    ViSession   instrSession,
    ViInt32 maximumTime
)
{
    ViStatus    error = VI_SUCCESS;
	maximumTime;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                              RSRTM20XX_ATTR_SEND_SW_TRIGGER,
                                              NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the waveform the oscilloscope acquires for the 
/// HIFN  channel you specify.  The waveform is from an acquisition that you 
/// HIFN  previously initiated.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array. The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm20xx_FetchWaveform(
    ViSession   instrSession,
    ViInt32 channel,
    ViInt32 waveformSize,
    ViReal64    waveformArray[],
    ViInt32*    actualPoints,
    ViReal64*   initialX,
    ViReal64*   xIncrement
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "CH%ld", channel);
    viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, buffer,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_START, initialX), 6, "Initial X");
    viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, buffer,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 7, "X Increment");

    sprintf (buffer, "FORM REAL,32;CHAN%ld:DATA?", channel);
    checkErr (rsrtm20xx_dataReadTrace (instrSession, buffer, waveformSize, waveformArray, actualPoints));
Error:
    if (p2data) free (p2data);
	viPrintf (instrSession, "FORM ASCII");
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the waveform the oscilloscope acquires for the 
/// HIFN  channel you specify.  The waveform is from an acquisition that you 
/// HIFN  previously initiated. To speed up the transmission, data are transfered as ViUInt16 array 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array. The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm20xx_FetchWaveformInteger (ViSession instrSession,
                                              ViInt32 channel,
                                              ViInt32 waveformSize,
                                              ViUInt16 waveformArray[],
                                              ViInt32 *actualPoints,
                                              ViReal64 *initialX,
                                              ViReal64 *xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar   *p2data  = NULL;
	ViChar   *ext_buf = NULL;
	ViUInt32 ret_cnt = 0;
	ViUInt32 count = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "CH%ld", channel);
    viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, buffer,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_START, initialX), 6, "Initial X");
    viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, buffer,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 7, "X Increment");

    viPrintf (instrSession, "FORM UINT,16;CHAN%ld:DATA?", channel);
	checkErr (rsrtm20xx_readBinaryDataBlock (instrSession, &ext_buf, &ret_cnt));
	count = ret_cnt/2;
	
	rsrtm20xx_SWAP_16 ((ViPBuf)ext_buf, count);
	
	*actualPoints = ( count > (ViUInt32) waveformSize) ? waveformSize : count;
	memcpy (waveformArray, ext_buf, *actualPoints * 2);
	
Error:
    if (p2data) free (p2data);
	viPrintf (instrSession, "FORM ASCII");
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the waveform the oscilloscope acquires for the 
/// HIFN  channel you specify.  The waveform is from an acquisition that you 
/// HIFN  previously initiated. To speed up the transmission, data are transfered as byte array 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array. The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/Indicates the length of time between points in the Waveform Array.  
ViStatus _VI_FUNC rsrtm20xx_FetchWaveformByte (ViSession instrSession,
                                           ViInt32 channel, ViInt32 waveformSize,
                                           ViByte waveformArray[],
                                           ViInt32 *actualPoints,
                                           ViReal64 *initialX,
                                           ViReal64 *xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar   *p2data  = NULL;
	ViChar   *ext_buf = NULL;
	ViUInt32 ret_cnt = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "CH%ld", channel);
    viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, buffer,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_START, initialX), 6, "Initial X");
    viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, buffer,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 7, "X Increment");

    viPrintf (instrSession, "FORM UINT,8;CHAN%ld:DATA?", channel);
	checkErr (rsrtm20xx_readBinaryDataBlock (instrSession, &ext_buf, &ret_cnt));
	
	*actualPoints = (ret_cnt > (ViUInt32) waveformSize) ? waveformSize : ret_cnt;
	memcpy (waveformArray, ext_buf, *actualPoints);
	
Error:
    if (p2data) free (p2data);
	viPrintf (instrSession, "FORM ASCII");
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns information on the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 1.
ViStatus _VI_FUNC rsrtm20xx_FetchWaveformHeader (ViSession instrSession,
                                             ViInt32 channel, ViReal64 *xStart,
                                             ViReal64 *xStop,
                                             ViInt32 *recordLength,
                                             ViInt32 *valuesPerSample)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    sprintf (header, "CHAN%ld:DATA:HEAD?\n", channel);
    checkErr (viWrite (instrSession, (ViBuf) header, (ViUInt32) strlen (header), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
    
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns the waveform conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm20xx_FetchWaveformConversionData (ViSession instrSession,
                                                     ViInt32 channel,
                                                     ViReal64 *xStart,
                                                     ViReal64 *xIncrement,
                                                     ViReal64 *yStart,
                                                     ViReal64 *yIncrement,
                                                     ViInt32 *yResolution)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
    checkErr (Rs_LockSession (instrSession, VI_NULL));    

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_START,
                                                 "Channel",
                                                 channel - 1,
                                                 15,
                                                 rep_cap), 2, "Channel");

	
    
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_START, xStart), 3, "X Start");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 4, "X Increment");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_Y_START, yStart), 5, "Y Start");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_Y_INCREMENT, yIncrement), 6, "Y Increment");
	viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_WAVEFORM_CONVERSION_Y_RESOLUTION, yResolution), 7, "Y Resolution");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the data of the envelope.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array.  The time 
/// HIPAR initialX/is relative to the trigger event.  
/// HIPAR xIncrement/This control is reserved.
ViStatus _VI_FUNC rsrtm20xx_FetchEnvelopeWaveform (ViSession instrSession,
                                                 ViInt32 channel,
                                                 ViInt32 waveformSize,
                                                 ViReal64 waveformArray[],
                                                 ViInt32 *actualPoints,
                                                 ViReal64 *initialX,
                                                 ViReal64 *xIncrement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
	xIncrement;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, channel, 1, 4), 2, "Channel");
   
    checkErr (rsrtm20xx_FetchEnvelopeWaveformHeader (instrSession, channel,
                                                 initialX, NULL,
                                                 actualPoints, NULL));
	
	sprintf (buffer, "CHAN%ld:DATA:ENV?", channel);
    checkErr (rsrtm20xx_dataReadTrace (instrSession, buffer, waveformSize, waveformArray, actualPoints));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns information on the envelope waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 2.
ViStatus _VI_FUNC rsrtm20xx_FetchEnvelopeWaveformHeader (ViSession instrSession,
                                                     ViInt32 channel,
                                                     ViReal64 *xStart,
                                                     ViReal64 *xStop,
                                                     ViInt32 *recordLength,
                                                     ViInt32 *valuesPerSample)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    sprintf (header, "CHAN%ld:DATA:ENV:HEAD?\n", channel);
    checkErr (viWrite (instrSession, (ViBuf) header, (ViUInt32) strlen (header), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
    
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns the envelope waveform conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm20xx_FetchEnvelopeWaveformConversionData (ViSession instrSession,
                                                             ViInt32 channel,
                                                             ViReal64 *xStart,
                                                             ViReal64 *xIncrement,
                                                             ViReal64 *yStart,
                                                             ViReal64 *yIncrement,
                                                             ViInt32 *yResolution)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
    checkErr (Rs_LockSession (instrSession, VI_NULL));    

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_START,
                                                 "Channel",
                                                 channel - 1,
                                                 15,
                                                 rep_cap), 2, "Channel");

	
    
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_START, xStart), 3, "X Start");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 4, "X Increment");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_Y_START, yStart), 5, "Y Start");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_Y_INCREMENT, yIncrement), 6, "Y Increment");
	viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_ENVELOPE_WAVEFORM_CONVERSION_Y_RESOLUTION, yResolution), 7, "Y Resolution");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the data of the math waveform points.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples, 
/// HIPAR waveformArray/or magnitudes of a spectrum.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsrtm20xx_FetchMathWaveform(
    ViSession    instrSession,
    ViInt32        mathNumber,
    ViInt32        waveformSize,
    ViReal64    waveformArray[],
    ViInt32*    actualPoints
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   header[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar   *p2data  = NULL;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

    // Use header to fetch horizontal parameters

    sprintf (header, "CALC:MATH%d:DATA?", mathNumber);
    checkErr (rsrtm20xx_dataReadTrace (instrSession, header, waveformSize, waveformArray, actualPoints));
    
    
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the header of math waveform data. The header contains  
/// HIFN  attributes of the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 1.
ViStatus _VI_FUNC rsrtm20xx_FetchMathWaveformHeader (ViSession instrSession,
                                                 ViInt32 mathNumber,
                                                 ViReal64 *xStart,
                                                 ViReal64 *xStop,
                                                 ViInt32 *recordLength,
                                                 ViInt32 *valuesPerSample)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf = NULL;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_MATH_WAVEFORM_HEADER,
                                                 "Math",
                                                 mathNumber - 1,
                                                 5,
                                                 rep_cap), 2, "Math Number");       

    viCheckParm (rsrtm20xx_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_MATH_WAVEFORM_HEADER,
                                                RSRTM20XX_IO_BUFFER_SIZE,
                                                buffer), 3, "Math Waveform Header");
    
    p2buf = strtok (buffer, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns the math waveform conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm20xx_FetchMathWaveformConversionData (ViSession instrSession,
                                                         ViInt32 channel,
                                                         ViReal64 *xStart,
                                                         ViReal64 *xIncrement,
                                                         ViReal64 *yStart,
                                                         ViReal64 *yIncrement,
                                                         ViInt32 *yResolution)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
    checkErr (Rs_LockSession (instrSession, VI_NULL));    

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_X_START,
                                                 "Math",
                                                 channel - 1,
                                                 15,
                                                 rep_cap), 2, "Channel");

	
    
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_X_START, xStart), 3, "X Start");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 4, "X Increment");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_Y_START, yStart), 5, "Y Start");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_Y_INCREMENT, yIncrement), 6, "Y Increment");
	viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_MATH_WAVEFORM_CONVERSION_Y_RESOLUTION, yResolution), 7, "Y Resolution");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the data of the reference waveform points.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR referenceWfm/Selects the reference waveform, the internal reference storage.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples, 
/// HIPAR waveformArray/or magnitudes of a spectrum.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsrtm20xx_FetchReferenceWaveform (ViSession instrSession,
                                                     ViInt32 referenceWfm,
                                                     ViInt32 waveformSize,
                                                     ViReal64 waveformArray[],
                                                     ViInt32 *actualPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;
    ViInt32  i = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, referenceWfm, 
            RSRTM20XX_VAL_REF_WAVEFORM_1, RSRTM20XX_VAL_REF_WAVEFORM_4), 2, "Reference Wfm");
    
    viCheckErr (viPrintf(instrSession, "REFC%ld:DATA?\n", referenceWfm));

    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    while (p2buf && (i < waveformSize))
    {
        if (i<waveformSize)
            waveformArray[i++] = atof (p2buf);
        else
            i++;
        p2buf = strtok (NULL, ",");
    }
    *actualPoints = i;
    
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the header of math waveform data. The header contains  
/// HIFN  attributes of the waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 1.
ViStatus _VI_FUNC rsrtm20xx_FetchReferenceWaveformHeader (ViSession instrSession,
                                                      ViInt32 channel,
                                                      ViReal64 *xStart,
                                                      ViReal64 *xStop,
                                                      ViInt32 *recordLength,
                                                      ViInt32 *valuesPerSample)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar      *p2buf = NULL;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_REFERENCE_WAVEFORM_HEADER,
                                                 "ReferenceWfm",
                                                 channel - 1,
                                                 5,
                                                 rep_cap), 2, "Reference Wfm");       

    viCheckParm (rsrtm20xx_GetAttributeViString (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_REFERENCE_WAVEFORM_HEADER,
                                                RSRTM20XX_IO_BUFFER_SIZE,
                                                buffer), 3, "Reference Waveform Header");
    
    p2buf = strtok (buffer, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN Returns the reference waveform conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel number.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm20xx_FetchReferenceWaveformConversionData (ViSession instrSession,
                                                              ViInt32 channel,
                                                              ViReal64 *xStart,
                                                              ViReal64 *xIncrement,
                                                              ViReal64 *yStart,
                                                              ViReal64 *yIncrement,
                                                              ViInt32 *yResolution)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[15] = "";
    checkErr (Rs_LockSession (instrSession, VI_NULL));    

    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_X_START,
                                                 "ReferenceWfm",
                                                 channel - 1,
                                                 15,
                                                 rep_cap), 2, "Channel"); 
    
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_X_START, xStart), 3, "X Start");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_X_INCREMENT, xIncrement), 4, "X Increment");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_Y_START, yStart), 5, "Y Start");
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
        RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_Y_INCREMENT, yIncrement), 6, "Y Increment");
	viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, rep_cap,
        RSRTM20XX_ATTR_REFERENCE_WAVEFORM_CONVERSION_Y_RESOLUTION, yResolution), 7, "Y Resolution");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the mask data. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the vertical values - voltages of recorded waveform samples, 
/// HIPAR waveformArray/or magnitudes of a spectrum.  
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform 
/// HIPAR actualPoints/Array parameter.
ViStatus _VI_FUNC rsrtm20xx_FetchMaskData (ViSession instrSession,
                                           ViInt32 waveformSize,
                                           ViReal64 waveformArray[],
                                           ViInt32 *actualPoints,
                                           ViReal64 *initialX,
                                           ViReal64 *xIncrement)
{
	ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, buffer,
        RSRTM20XX_ATTR_MASK_DATA_CONVERSION_X_START, initialX), 6, "Initial X");
    viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, buffer,
        RSRTM20XX_ATTR_MASK_DATA_CONVERSION_X_INCREMENT, xIncrement), 7, "X Increment");

    sprintf (buffer, "FORM REAL,32;MASK:DATA?");
    checkErr (rsrtm20xx_dataReadTrace (instrSession, buffer, waveformSize, waveformArray, actualPoints));
Error:
    if (p2data) free (p2data);
	viPrintf (instrSession, "FORM ASCII");
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns information of the mask data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 2.
ViStatus _VI_FUNC rsrtm20xx_FetchMaskDataHeader(
	ViSession	instrSession,
	ViReal64*	xStart,
	ViReal64*	xStop,
	ViInt32*	recordLength,
	ViInt32*	valuesPerSample
)
{
	ViStatus error = VI_SUCCESS;
    ViChar   header[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
    sprintf (header, "MASK:DATA:HEAD?\n");
    checkErr (viWrite (instrSession, (ViBuf) header, (ViUInt32) strlen (header), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsrtm20xx_CheckStatus (instrSession));
    
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  Returns the mask conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the 
/// HIPAR xIncrement/indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm20xx_FetchMaskConversionData(
	ViSession	instrSession,
	ViReal64*	xStart,
	ViReal64*	xIncrement,
	ViReal64*	yStart,
	ViReal64*	yIncrement,
	ViInt32*	yResolution
)
{
	ViStatus	error = VI_SUCCESS;
    checkErr (Rs_LockSession (instrSession, VI_NULL)); 
	
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
        RSRTM20XX_ATTR_MASK_DATA_CONVERSION_X_START, xStart), 2, "X Start");
	
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
        RSRTM20XX_ATTR_MASK_DATA_CONVERSION_X_INCREMENT, xIncrement), 3, "X Increment");
	
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
        RSRTM20XX_ATTR_MASK_DATA_CONVERSION_Y_START, yStart), 4, "Y Start");
	
	viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
        RSRTM20XX_ATTR_MASK_DATA_CONVERSION_Y_INCREMENT, yIncrement), 5, "Y Increment");
	
	viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
        RSRTM20XX_ATTR_MASK_DATA_CONVERSION_Y_RESOLUTION, yResolution), 6, "Y Resolution");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;    
}

/// HIFN  This function obtains a main waveform measurement on the channel you 
/// HIFN  specify, and returns the measurement value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR measurement/Selects waveform measurement.
/// HIPAR measurementFunction/Defines or queries the main measurement.
/// HIPAR measurementResult/Returns the measured value.  
ViStatus _VI_FUNC rsrtm20xx_FetchMainWaveformMeasurement (ViSession instrSession,
                                                           ViInt32 measurement,
                                                           ViInt32 measurementFunction,
                                                           ViReal64 *measurementResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rep_cap[75] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_FREQUENCY,
                                                 "Measurement",
                                                 measurement - 1,
                                                 25,
                                                 rep_cap), 2, "Measurement");

    
	switch (measurementFunction)
	{
		case RSRTM20XX_VAL_MEAS_FREQUENCY:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_FREQUENCY, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_PERIOD:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_PERIOD, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_PEAK:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_PEAK, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_UPEAK:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_MAXIMUM_PEAK, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_LPEAK:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_MINIMUM_PEAK, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_PPC:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_POSITIVE_PULSES, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_NPC:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_NEGATIVE_PULSES, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_REC:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_RISING_EDGES, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_FEC:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_FALLING_EDGES, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_HIGH:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_HIGH_LEVEL, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_LOW:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_LOW_LEVEL, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_AMP:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_AMPLITUDE, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_MEAN:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_MEAN, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_RMS:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_RMS, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_RTM:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_RISE_TIME, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_FTIM:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_FALLING_TIME, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_PDCY:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_POSITIVE_CYCLE, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_NDCY:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_NEGATIVE_CYCLE, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_PPW:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_WIDTH_POSITIVE_PULSE, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_NPW:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_WIDTH_NEGATIVE_PULSE, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_CYCM:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_MEAN_VALUE, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_CYCR:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_RMS_VALUE, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_SSTDD:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_STANDARD_DEVIATIONS, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_TFR:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_TRIGGER_FREQUENCY, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_TPER:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_TRIGGER_PERIOD, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_DEL:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_DELAY, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_PHA:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_PHASE, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_BWID:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_BURST_WIDTH, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_CYCS:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_STANDARD_DEVIATION_VALUE, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_POV:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_POSITIVE_OVERSHOOT, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_NOV:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_NEGATIVE_OVERSHOOT, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_TBFR:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_B_TRIGGER_FREQUENCY, 
											measurementResult), 4, "Result");
			break;
		case RSRTM20XX_VAL_MEAS_TBP:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, rep_cap,
						        			RSRTM20XX_ATTR_MAIN_MEASUREMENT_RESULT_B_TRIGGER_PERIOD, 
											measurementResult), 4, "Result");
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Measurement Function");
			break;
	}
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error; 
}  

/// HIFN  This function aborts an acquisition and returns the oscilloscope to 
/// HIFN  the Idle state.  You initiate an acquisition with the 
/// HIFN  rsrtm20xx_InitiateAcquisition function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
ViStatus _VI_FUNC rsrtm20xx_Abort(
    ViSession   instrSession
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                              RSRTM20XX_ATTR_STOP, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN This function enables the power analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/enables the power analysis.
ViStatus _VI_FUNC rsrtm20xx_ConfigurePowerAnalysisState (ViSession instrSession,
                                                ViBoolean state)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_SetAttributeViBoolean (instrSession, "",
						        			RSRTM20XX_ATTR_POWER_ANALYSIS_ENABLED, 
											state), 2, "state");
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error; 
}

/// HIFN  This function configures power analysis type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects power analysis type.
ViStatus _VI_FUNC rsrtm20xx_ConfigurePowerAnalysisType(
	ViSession	instrSession,
	ViInt32	type
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, "",
						        			RSRTM20XX_ATTR_POWER_ANALYSIS_TYPE, 
											type), 2, "type");
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;    
}


/// HIFN  This function resets statistics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_ResetPowerAnalysisStatistics(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                            	RSRTM20XX_ATTR_POWER_ANALYSIS_RESET_STATISTICS, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;    
}


/// HIFN  This function starts the automatic deskew procedure to align the 
/// HIFN  waveforms of all visible channels. It is necessary to deskew if a 
/// HIFN  current and a voltage probe is used in the measurment. Use the R&S 
/// HIFN  RT-ZF20 power deskew fixture to deskew the probes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerAnalysisDeskew(
	ViSession	instrSession,
	ViInt32 	maximumTime
)
{
	ViStatus    error = VI_SUCCESS;
	ViInt32 	oldTimeout = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rsrtm20xx_GetOPCTimeout(instrSession, &oldTimeout));
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, maximumTime));

	viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                            	RSRTM20XX_ATTR_POWER_ANALYSIS_START_DESKEW, NULL));
	
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, oldTimeout));
    
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error; 
}


/// HIFN  This function sets zero offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_SetPowerAnalysisZeroOffset(
	ViSession	instrSession,
	ViInt32 	maximumTime
)
{
	ViStatus    error = VI_SUCCESS;
	ViInt32 	oldTimeout = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rsrtm20xx_GetOPCTimeout(instrSession, &oldTimeout));
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, maximumTime));

	viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                            	RSRTM20XX_ATTR_POWER_ANALYSIS_ZERO_OFFSET, NULL));
	
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, oldTimeout));
    
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;  
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerAnalysisAddReport (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_ANALYSIS_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function starts the power quality measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerQualityMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_QUALITY_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerQualityMeasurementAddReport (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_QUALITY_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error; 
}

/// HIFN  Returns the statistic value of the specified measurement series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects quality type.
/// HIPAR statistics/Selects statistic value.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerQualityMeasurementVoltageResult(
	ViSession	instrSession,
	ViInt32	type,
	ViInt32	statistics,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      cmd[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K31")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_QUALITY_TYPE_RMS, 
											RSRTM20XX_VAL_QUALITY_TYPE_CRES), 2, "Type");
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, statistics, RSRTM20XX_VAL_STATISTICS_ACT, 
												RSRTM20XX_VAL_STATISTICS_WFMC), 3, "Statistics");
      
    sprintf (cmd, "POW:QUAL:RES:VOLT:%s:%s?", qualityTypeArr[type], statisticsArr[statistics]);
    viCheckErr (rsrtm20xx_QueryViReal64 (instrSession, cmd, measurementResult));
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  Returns the statistic value of the specified measurement series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects quality type.
/// HIPAR statistics/Selects statistic value.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerQualityMeasurementCurrentResult(
	ViSession	instrSession,
	ViInt32	type,
	ViInt32	statistics,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      cmd[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K31")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_QUALITY_TYPE_RMS, 
											RSRTM20XX_VAL_QUALITY_TYPE_CRES), 2, "Type");
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, statistics, RSRTM20XX_VAL_STATISTICS_ACT, 
												RSRTM20XX_VAL_STATISTICS_WFMC), 3, "Statistics");
    
    sprintf (cmd, "POW:QUAL:RES:CURR:%s:%s?", qualityTypeArr[type], statisticsArr[statistics]);
    viCheckErr (rsrtm20xx_QueryViReal64 (instrSession, cmd, measurementResult));
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  Returns the statistic value of the specified measurement series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects quality type.
/// HIPAR statistics/Selects statistic value.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerQualityMeasurementPowerResult(
	ViSession	instrSession,
	ViInt32	type,
	ViInt32	statistics,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      cmd[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K31")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_QUALITY_TYPE_POWER_REAL, 
											RSRTM20XX_VAL_QUALITY_TYPE_POWER_PHAS), 2, "Type");
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, statistics, RSRTM20XX_VAL_STATISTICS_ACT, 
												RSRTM20XX_VAL_STATISTICS_WFMC), 3, "Statistics");
    
    sprintf (cmd, "POW:QUAL:RES:POW:%s:%s?", qualityTypePowerArr[type], statisticsArr[statistics]);
    viCheckErr (rsrtm20xx_QueryViReal64 (instrSession, cmd, measurementResult));
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  This function starts the power consumption measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerConsumptionMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_CONSUMPTION_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}


/// HIFN  This function restarts the power consumption measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_RestartPowerConsumptionMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_RESTART_POWER_CONSUMPTION_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerConsumptionMeasurementAddReport (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_CONSUMPTION_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;    
}

/// HIFN  Returns the statistic value of the specified measurement series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects consumption type.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerConsumptionMeasurementResults(
	ViSession	instrSession,
	ViInt32	type,
	ViReal64*	measurementResult
)
{
    ViStatus error = VI_SUCCESS; 
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    switch (type)
    {
        case RSRTM20XX_VAL_CONSUMPTION_TYPE_DURATION:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_CONSUMPTION_MEASUREMENT_DURATION_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;                
        case RSRTM20XX_VAL_CONSUMPTION_TYPE_ENERGY:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_CONSUMPTION_MEASUREMENT_ENERGY_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
		case RSRTM20XX_VAL_CONSUMPTION_TYPE_ACTIVE:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_CONSUMPTION_MEASUREMENT_ACTIVE_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Type");
    }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  This function starts the power current harmonics measurement.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_START_POWER_CURRENT_HARMONICS_MEASUREMENT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     POWer:HARMonics:EXECute
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerCurrentHarmonicsMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_CURRENT_HARMONICS_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;  	    
}

/// HIFN  Returns the result of the selected measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerCurrentHarmonicsMeasurementAddReport (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error; 
}

/// HIFN  Returns the result of the selected measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR resultNumber/Result number.
/// HIPAR type/Selects harmonics type.
/// HIPAR measurementResult/Returns the result of the selected measurement.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerCurrentHarmonicsMeasurementResult(
	ViSession	instrSession,
	ViInt32	resultNumber,
	ViInt32	type,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      cmd[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K31")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, resultNumber, 1, 40), 2, "Result Number");
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_HARMONICS_TYPE_VIOLENT_COUNT, 
												RSRTM20XX_VAL_HARMONICS_TYPE_WFMC), 3, "Statistics");
    
    sprintf (cmd, "POW:HARM:RES%ld:%s?", resultNumber, harmonicsTypeArr[type]);
    viCheckErr (rsrtm20xx_QueryViReal64 (instrSession, cmd, measurementResult));
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
}


/// HIFN  Returns the limit check result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR resultNumber/Result number.
/// HIPAR limitCheck/Returns the limit check result.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerCurrentHarmonicsMeasurementLimitCheckResult(
	ViSession	instrSession,
	ViInt32	resultNumber,
	ViInt32*	limitCheck
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar   rep_cap[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_LIMIT_CHECK_RESULT,
                                                 "ResultNumber",
                                                 resultNumber - 1,
                                                 25,
                                                 rep_cap), 2, "ResultNumber");
	
	viCheckParm(rsrtm20xx_GetAttributeViInt32 (instrSession, rep_cap,
		        		RSRTM20XX_ATTR_POWER_CURRENT_HARMONICS_MEASUREMENT_LIMIT_CHECK_RESULT, 
						limitCheck), 3, "Limit Check"); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
}


/// HIFN  This function resets the results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR resultNumber/Result number.
ViStatus _VI_FUNC rsrtm20xx_ResetPowerCurrentHarmonicsMeasurementResults(
	ViSession	instrSession,
	ViInt32	resultNumber
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar   rep_cap[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_RESET_POWER_CURRENT_HARMONICS_MEASUREMENT_RESULTS,
                                                 "ResultNumber",
                                                 resultNumber - 1,
                                                 25,
                                                 rep_cap), 2, "ResultNumber");
	
	viCheckErr(rsrtm20xx_SetAttributeViString(instrSession, rep_cap,
		        		RSRTM20XX_ATTR_RESET_POWER_CURRENT_HARMONICS_MEASUREMENT_RESULTS, 
						VI_NULL)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;      
}


/// HIFN  This function starts the power inrush current measurement.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_START_POWER_INRUSH_CURRENT_MEASUREMENT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     POWer:INRushcurrent:EXECute
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerInrushCurrentMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_INRUSH_CURRENT_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerInrushCurrentMeasurementAddReport (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;

}

/// HIFN  Returns the result of the selected measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR gateNumber/Gate number.
/// HIPAR type/Selects inrush current type.
/// HIPAR measurementResult/Returns the result of the selected measurement.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerInrushCurrentMeasurementResult(
	ViSession	instrSession,
	ViInt32	gateNumber,
	ViInt32	type,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar   rep_cap[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_AREA_RESULT,
                                                 "Gate",
                                                 gateNumber - 1,
                                                 25,
                                                 rep_cap), 2, "Gate"); 
    switch (type)
    {
        case RSRTM20XX_VAL_INRUSH_CURRENT_TYPE_AREA:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_AREA_RESULT, 
						measurementResult), 4, "Measurement Result");
            break;                
        case RSRTM20XX_VAL_INRUSH_CURRENT_TYPE_MAXIMUM:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_INRUSH_CURRENT_MEASUREMENT_MAXIMUM_RESULT, 
						measurementResult), 4, "Measurement Result");
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");
    }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  This function starts the power ripple measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerRippleMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_RIPPLE_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;      
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerRippleMeasurementAddReport (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_RIPPLE_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;

}

/// HIFN  Returns the statistic value of the specified measurement series.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     POWer:RIPPle:RESult:FREQuency[:ACTual]
/// HIFN     POWer:RIPPle:RESult:FREQuency:AVG
/// HIFN     POWer:RIPPle:RESult:FREQuency:STDDev
/// HIFN     POWer:RIPPle:RESult:FREQuency:NPEak
/// HIFN     POWer:RIPPle:RESult:FREQuency:PPEak
/// HIFN     POWer:RIPPle:RESult:FREQuency:WFMCount
/// HIFN     POWer:RIPPle:RESult:PEAK[:ACTual]
/// HIFN     POWer:RIPPle:RESult:PEAK:AVG
/// HIFN     POWer:RIPPle:RESult:PEAK:STDDev
/// HIFN     POWer:RIPPle:RESult:PEAK:NPEak
/// HIFN     POWer:RIPPle:RESult:PEAK:PPEak
/// HIFN     POWer:RIPPle:RESult:PEAK:WFMCount
/// HIFN     POWer:RIPPle:RESult:STDDev[:ACTual]
/// HIFN     POWer:RIPPle:RESult:STDDev:AVG
/// HIFN     POWer:RIPPle:RESult:STDDev:STDDev
/// HIFN     POWer:RIPPle:RESult:STDDev:NPEak
/// HIFN     POWer:RIPPle:RESult:STDDev:PPEak
/// HIFN     POWer:RIPPle:RESult:STDDev:WFMCount
/// HIFN     POWer:RIPPle:RESult:UPEak[:ACTual]
/// HIFN     POWer:RIPPle:RESult:UPEak:AVG
/// HIFN     POWer:RIPPle:RESult:UPEak:STDDev
/// HIFN     POWer:RIPPle:RESult:UPEak:NPEak
/// HIFN     POWer:RIPPle:RESult:UPEak:PPEak
/// HIFN     POWer:RIPPle:RESult:UPEak:WFMCount
/// HIFN     POWer:RIPPle:RESult:LPEak[:ACTual]
/// HIFN     POWer:RIPPle:RESult:LPEak:AVG
/// HIFN     POWer:RIPPle:RESult:LPEak:STDDev
/// HIFN     POWer:RIPPle:RESult:LPEak:NPEak
/// HIFN     POWer:RIPPle:RESult:LPEak:PPEak
/// HIFN     POWer:RIPPle:RESult:LPEak:WFMCount
/// HIFN     POWer:RIPPle:RESult:MEAN[:ACTual]
/// HIFN     POWer:RIPPle:RESult:MEAN:AVG
/// HIFN     POWer:RIPPle:RESult:MEAN:STDDev
/// HIFN     POWer:RIPPle:RESult:MEAN:NPEak
/// HIFN     POWer:RIPPle:RESult:MEAN:PPEak
/// HIFN     POWer:RIPPle:RESult:MEAN:WFMCount
/// HIFN     POWer:RIPPle:RESult:PERiod[:ACTual]
/// HIFN     POWer:RIPPle:RESult:PERiod:AVG
/// HIFN     POWer:RIPPle:RESult:PERiod:STDDev
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects ripple type.
/// HIPAR statistics/Selects statistic value.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerRippleMeasurementResult(
	ViSession	instrSession,
	ViInt32	type,
	ViInt32	statistics,
	ViReal64*	measurementResult
)
{
 	ViStatus    error = VI_SUCCESS;
	ViChar      cmd[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K31")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_RIPPLE_TYPE_FREQUENCY, 
											RSRTM20XX_VAL_RIPPLE_TYPE_NDCYCLE), 2, "Type");
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, statistics, RSRTM20XX_VAL_STATISTICS_ACT, 
												RSRTM20XX_VAL_STATISTICS_WFMC), 2, "Statistics");
    
    sprintf (cmd, "POW:RIPP:RES:%s:%s?", rippleTypeArr[type], statisticsArr[statistics]);
    viCheckErr (rsrtm20xx_QueryViReal64 (instrSession, cmd, measurementResult));
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  This function starts the power spectrum measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerSpectrumMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_SPECTRUM_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}


/// HIFN  Returns the result of the selected measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR resultNumber/Result number.
/// HIPAR type/Selects spectrum measurement type.
/// HIPAR measurementResult/Returns the result of the selected measurement.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSpectrumMeasurementResult(
	ViSession	instrSession,
	ViInt32	resultNumber,
	ViInt32	type,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      cmd[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K31")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, resultNumber, 1, 40), 2, "Result Number");
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_SPECTRUM_TYPE_FREQUENCY, 
												RSRTM20XX_VAL_SPECTRUM_TYPE_WFMC), 3, "Statistics");
    
    sprintf (cmd, "POW:SPEC:RES%ld:%s?", resultNumber, spectrumTypeArr[type]);
    viCheckErr (rsrtm20xx_QueryViReal64 (instrSession, cmd, measurementResult));
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerSpectrumMeasurementAddReport (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_SPECTRUM_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function resets the results.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_RESET_POWER_SPECTRUM_MEASUREMENT_RESULTS
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     POWer:SPECtrum:RESult<n>:RESet
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR resultNumber/Result number.
ViStatus _VI_FUNC rsrtm20xx_ResetPowerSpectrumMeasurementResults(
	ViSession	instrSession,
	ViInt32	resultNumber
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar   rep_cap[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_RESET_POWER_SPECTRUM_MEASUREMENT_RESULTS,
                                                 "ResultNumber",
                                                 resultNumber - 1,
                                                 25,
                                                 rep_cap), 2, "ResultNumber");
	
	viCheckErr(rsrtm20xx_SetAttributeViString(instrSession, rep_cap,
		        		RSRTM20XX_ATTR_RESET_POWER_SPECTRUM_MEASUREMENT_RESULTS, VI_NULL)); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
}


/// HIFN  This function starts the slew rate measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerSlewRateMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_SLEW_RATE_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerSlewRateMeasurementAddReport (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_SLEW_RATE_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error; 
}

/// HIFN  Returns the statistic value of the specified measurement series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects slew rate measurement type.
/// HIPAR statistics/Selects statistic value.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSlewRateMeasurementResult(
	ViSession	instrSession,
	ViInt32	type,
	ViInt32	statistics,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      cmd[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K31")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_SLEW_RATE_TYPE_LPEAK, 
											RSRTM20XX_VAL_SLEW_RATE_TYPE_UPEAK), 2, "Type");
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, statistics, RSRTM20XX_VAL_STATISTICS_ACT, 
												RSRTM20XX_VAL_STATISTICS_WFMC), 3, "Statistics");
    
    sprintf (cmd, "POW:SLEW:RES:%s:%s?", slewRateTypeArr[type], statisticsArr[statistics]);
    viCheckErr (rsrtm20xx_QueryViReal64 (instrSession, cmd, measurementResult));
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  This function starts the power modulation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerModulationMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_MODULATION_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerModulationMeasurementAddReport (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_MODULATION_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}

/// HIFN  Returns the statistic value of the specified measurement series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects modulation measurement type.
/// HIPAR statistics/Selects statistic value.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerModulationMeasurementResult(
	ViSession	instrSession,
	ViInt32	type,
	ViInt32	statistics,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      cmd[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K31")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_MODULATION_RESULT_TYPE_LPE, 
											RSRTM20XX_VAL_MODULATION_RESULT_TYPE_STDD), 2, "Type");
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, statistics, RSRTM20XX_VAL_STATISTICS_ACT, 
												RSRTM20XX_VAL_STATISTICS_WFMC), 3, "Statistics");
    
    sprintf (cmd, "POW:MOD:RES:%s:%s?", modulationTypeArr[type], statisticsArr[statistics]);
    viCheckErr (rsrtm20xx_QueryViReal64 (instrSession, cmd, measurementResult));
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  This function starts the power dynamic on resistance measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerDynamicOnResistanceMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_DYNAMIC_ON_RESISTANCE_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerDynamicOnResistanceMeasurementAddReport (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_DYNAMIC_ON_RESISTANCE_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  Returns the result of the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerDynamicOnResistanceMeasurementResult(
	ViSession	instrSession,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm(rsrtm20xx_GetAttributeViReal64 (instrSession, "",
		        		RSRTM20XX_ATTR_POWER_DYNAMIC_ON_RESISTANCE_MEASUREMENT_RESULT, 
						measurementResult), 2, "Measurement Result"); 
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
}


/// HIFN  This function starts the power efficiency measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerEfficiencyMeasurement(
	ViSession	instrSession
)
{
 	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_EFFICIENCY_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerEfficiencyMeasurementAddReport (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_EFFICIENCY_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  Returns the statistic value of the specified measurement series.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     POWer:EFFiciency:RESult:INPut:REALpower[:ACTual]
/// HIFN     POWer:EFFiciency:RESult:INPut:REALpower:AVG
/// HIFN     POWer:EFFiciency:RESult:INPut:REALpower:STDDev
/// HIFN     POWer:EFFiciency:RESult:INPut:REALpower:NPEak
/// HIFN     POWer:EFFiciency:RESult:INPut:REALpower:PPEak
/// HIFN     POWer:EFFiciency:RESult:INPut:REALpower:WFMCount
/// HIFN     POWer:EFFiciency:RESult:OUTPut:REALpower[:ACTual]
/// HIFN     POWer:EFFiciency:RESult:OUTPut:REALpower:AVG
/// HIFN     POWer:EFFiciency:RESult:OUTPut:REALpower:STDDev
/// HIFN     POWer:EFFiciency:RESult:OUTPut:REALpower:NPEak
/// HIFN     POWer:EFFiciency:RESult:OUTPut:REALpower:PPEak
/// HIFN     POWer:EFFiciency:RESult:OUTPut:REALpower:WFMCount
/// HIFN     POWer:EFFiciency:RESult:EFFiciency[:ACTual]
/// HIFN     POWer:EFFiciency:RESult:EFFiciency:AVG
/// HIFN     POWer:EFFiciency:RESult:EFFiciency:STDDev
/// HIFN     POWer:EFFiciency:RESult:EFFiciency:NPEak
/// HIFN     POWer:EFFiciency:RESult:EFFiciency:PPEak
/// HIFN     POWer:EFFiciency:RESult:EFFiciency:WFMCount
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects efficiency measurement type.
/// HIPAR statistics/Selects statistic value.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerEfficiencyMeasurementResult(
	ViSession	instrSession,
	ViInt32	type,
	ViInt32	statistics,
	ViReal64*	measurementResult
)
{
	ViStatus    error = VI_SUCCESS;
	ViChar      cmd[RSRTM20XX_IO_BUFFER_SIZE];
	ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE];
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    checkErr (rsrtm20xx_GetAttributeViString (instrSession, "", RS_ATTR_OPTIONS_LIST, RSRTM20XX_IO_BUFFER_SIZE, buffer));
    
    if (!(rsrtm20xx_CheckOption (instrSession,"K31")))
    {
        viCheckErr (RS_ERROR_INSTRUMENT_OPTION);
    }
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_EFFICIENCY_TYPE_EFFICIENCY, 
											RSRTM20XX_VAL_EFFICIENCY_TYPE_OUTPUT), 2, "Type");
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, statistics, RSRTM20XX_VAL_STATISTICS_ACT, 
												RSRTM20XX_VAL_STATISTICS_WFMC), 3, "Statistics");
    
    sprintf (cmd, "POW:EFF:RES:%s:%s?", efficiencyTypeArr[type], statisticsArr[statistics]);
    viCheckErr (rsrtm20xx_QueryViReal64 (instrSession, cmd, measurementResult));
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error; 	    
}


/// HIFN  This function starts the power switchingloss measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerSwitchinglossMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_SWITCHINGLOSS_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}


/// HIFN  This function sets gate to waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_SetPowerSwitschinglossMeasurementGateToWaveform(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_SET_POWER_SWITSCHINGLOSS_MEASUREMENT_GATE_TO_WAVEFORM, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerSwitchinglossMeasurementAddReport (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}

/// HIFN  Returns the result of the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects switchingloss measurement type.
/// HIPAR measurementResult/Returns the result of the measurement.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSwitchinglossMeasurementEnergyResult(
	ViSession	instrSession,
	ViInt32	type,
	ViReal64*	measurementResult
)
{
	ViStatus error = VI_SUCCESS; 
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    switch (type)
    {
        case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_CONDUCTION:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_CONDUCTION_ENERGY_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;                
        case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_NO_COND:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_NO_CONDUCTION_ENERGY_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
		case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_TURN_ON:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_ON_ENERGY_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
		case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_TURN_OFF:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_OFF_ENERGY_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
		case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_TOTAL:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TOTAL_ENERGY_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Type");
		break;
    }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  Returns the result of the measurement.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_CONDUCTION_POWER_RESULT
/// HIFN     
/// HIFN  RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_NO_CONDUCTION_POWER_RESULT
/// HIFN     RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_ON_POWER_RESULT
/// HIFN     RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_OFF_POWER_RESULT
/// HIFN     RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TOTAL_POWER_RESULT
/// HIFN     
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     POWer:SWITching:RESult:CONDuction:POWer?
/// HIFN     POWer:SWITching:RESult:NCONduction:POWer?
/// HIFN     POWer:SWITching:RESult:TON:POWer?
/// HIFN     POWer:SWITching:RESult:TOFF:POWer?
/// HIFN     POWer:SWITching:RESult:TOTal:POWer?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects switchingloss measurement type.
/// HIPAR measurementResult/Returns the result of the measurement.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSwitchinglossMeasurementPowerResult(
	ViSession	instrSession,
	ViInt32	type,
	ViReal64*	measurementResult
)
{
	ViStatus error = VI_SUCCESS; 
    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    switch (type)
    {
        case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_CONDUCTION:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_CONDUCTION_POWER_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;                
        case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_NO_COND:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_NO_CONDUCTION_POWER_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
		case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_TURN_ON:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_ON_POWER_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
		case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_TURN_OFF:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TURN_OFF_POWER_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
		case RSRTM20XX_VAL_SWITCHINGLOSS_TYPE_TOTAL:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
		        		RSRTM20XX_ATTR_POWER_SWITCHINGLOSS_MEASUREMENT_TOTAL_POWER_RESULT, 
						measurementResult), 3, "Measurement Result");
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Type");
		break;
    }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
} 

/// HIFN  This function starts the power on off measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerOnOffMeasurement(
	ViSession	instrSession
)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_ON_OFF_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;    
}

/// HIFN  This function adds a power report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerOnOffMeasurementAddReport (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_ON_OFF_MEASUREMENT_ADD_REPORT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  Returns the result of the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurementResult/Returns the statistic value of the specified measurement series.
/// HIPAR resultNumber/Result number.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerOnOffMeasurementResult (ViSession instrSession,
                                                     ViInt32 resultNumber,
                                                     ViReal64 *measurementResult)
{
	ViStatus    error = VI_SUCCESS;
	ViChar   rep_cap[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rsrtm20xx_GetAttributeRepCapName (instrSession,
                                                 RSRTM20XX_ATTR_POWER_ON_OFF_MEASUREMENT_RESULT,
                                                 "OnOffResult",
                                                 resultNumber - 1,
                                                 25,
                                                 rep_cap), 2, "OnOffResult");
	
	viCheckParm(rsrtm20xx_GetAttributeViReal64 (instrSession, rep_cap,
		        		RSRTM20XX_ATTR_POWER_ON_OFF_MEASUREMENT_RESULT, 
						measurementResult), 3, "Measurement Result"); 
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function starts the safe operating area measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerSafeOperatingAreaMeasurement (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_SAFE_OPERATING_AREA_MEASUREMENT, NULL));
	
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error; 
}

/// HIFN  This function restarts the safe operating area measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_RestartPowerSafeOperatingAreaMeasurement (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_RESTART_POWER_SAFE_OPERATING_AREA_MEASUREMENT, NULL));
	
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaAddToReport (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_ADD_REPORT, NULL));
	
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function sets the safe operation area scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR scale/Selects the scale for the result diagram.
/// HIPAR maskScale/Sets the scale for the mask, linear or logarithmic.
/// HIPAR displayScale/Sets a linear or a logarithmic scaled for the displayed results.
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaScaling (ViSession instrSession,
                                                           ViInt32 scale,
                                                           ViInt32 maskScale,
                                                           ViInt32 displayScale)
{
	ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, NULL,
        RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_SCALE, scale), 2, "Scale");    
    
	viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, NULL,
        RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_MASK_SCALE, maskScale), 3, "Mask Scale");    
	
	viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, NULL,
        RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_DISPLAY_SCALE, displayScale), 4, "Display Scale");    
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;

}

/// HIFN  This function manages the safe operating area linear point measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR operation/Selects the operation.
/// HIPAR pointNumber/Sets the point number.
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaLinPointManagement (ViSession instrSession,
                                                                      ViInt32 operation,
                                                                      ViInt32 pointNumber)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, operation, RSRTM20XX_VAL_ADD, RSRTM20XX_VAL_INSERT), 2, "Operation");
	
	switch (operation)
	{
		case RSRTM20XX_VAL_ADD:
                
			    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                            RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LIN_ADD, NULL));
		 break;
		 
		 case RSRTM20XX_VAL_INSERT:
			 
	            viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, NULL,
                            RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_INSERT, pointNumber), 3, "Point Number");
		 break;
		 
		 case RSRTM20XX_VAL_REMOVE:
			 
	            viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, NULL,
                            RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_REMOVE, pointNumber), 3, "Point Number");  
		 
		 break;
	
	
		 default:
			    viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
		 break;
	 }
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
	
}

/// HIFN  This function sets the voltage/current value of the SOA point
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the value type.
/// HIPAR value/Sets the point value.
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaLinPointValue (ViSession instrSession,
                                                                 ViInt32 pointNumber,
                                                                 ViInt32 valueType,
                                                                 ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, valueType, RSRTM20XX_VAL_POWER_SOURCE_CURRENT_MIN, RSRTM20XX_VAL_POWER_SOURCE_VOLTAGE), 3, "Value Type");
	
	viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, "",
	        RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_SELECTED_POINT, pointNumber), 2, "Point Number");
	
	switch (valueType)
    {
        case RSRTM20XX_VAL_POWER_SOURCE_CURRENT_MIN:
            viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_POINT_CURRENT_MIN,
											   value), 4, "Value");
            break;                
        case RSRTM20XX_VAL_POWER_SOURCE_CURRENT_MAX:
            viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_POINT_CURRENT_MAX,
											   value), 4, "Value");
            break;
		case RSRTM20XX_VAL_POWER_SOURCE_VOLTAGE:
            viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_POINT_VOLTAGE,
											   value), 4, "Value");
            break;
		
            default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value Type");
		    break;
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Queries the number of points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR count/Returns the number of points
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSafeOperatingAreaLinCount (ViSession instrSession,
                                                                 ViInt32 *count)
{
	ViStatus	error = VI_SUCCESS;

	viCheckParm(rsrtm20xx_GetAttributeViInt32 (instrSession, "", RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LINEAR_COUNT, 
						                       count), 2, "Count"); 
		        		
Error:
	Rs_UnlockSession(instrSession, VI_NULL);
	return error;
}

/// HIFN  This function starts the safe operating area measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR operation/Selects the operation.
/// HIPAR pointNumber/Sets the point number.
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaLogPointManagement (ViSession instrSession,
                                                                      ViInt32 operation,
                                                                      ViInt32 pointNumber)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, operation, RSRTM20XX_VAL_ADD, RSRTM20XX_VAL_INSERT), 2, "Operation");
	
	switch (operation)
	{
		case RSRTM20XX_VAL_ADD:
                
			    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                            RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_ADD, NULL));
		 break;
		 
		 case RSRTM20XX_VAL_INSERT:
			 
	            viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, "",
                            RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_INSERT, pointNumber), 3, "Point Number");
		 break;
		 
		 case RSRTM20XX_VAL_REMOVE:
			 
	            viCheckParm(rsrtm20xx_SetAttributeViInt32 (instrSession, "",
                            RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_REMOVE, pointNumber), 3, "Point Number");  
		 
		 break;
	
	
		 default:
			    viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
		 break;
	 }
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the voltage/current value of the SOA point
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular instrument session. 
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the value type.
/// HIPAR value/Sets the point value.
ViStatus _VI_FUNC rsrtm20xx_PowerSafeOperatingAreaLogPointValue (ViSession instrSession,
                                                                 ViInt32 pointNumber,
                                                                 ViInt32 valueType,
                                                                 ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, valueType, RSRTM20XX_VAL_POWER_SOURCE_CURRENT_MIN, RSRTM20XX_VAL_POWER_SOURCE_VOLTAGE), 3, "Value Type");

	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_SELECTED_POINT, pointNumber), 2, "Point Number");
	
	switch (valueType)
    {
        case RSRTM20XX_VAL_POWER_SOURCE_CURRENT_MIN:
            viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_POINT_CURRENT_MIN,
											   value), 4, "Value");
            break;                
        case RSRTM20XX_VAL_POWER_SOURCE_CURRENT_MAX:
            viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_POINT_CURRENT_MAX,
											   value), 4, "Value");
            break;
		case RSRTM20XX_VAL_POWER_SOURCE_VOLTAGE:
            viCheckParm(rsrtm20xx_SetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_POINT_VOLTAGE,
											   value), 4, "Value");
            break;
		
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value Type");
		break;
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Queries the number of points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR count/Returns the number of points.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSafeOperatingAreaLogCount (ViSession instrSession,
                                                                 ViInt32 *count)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_GetAttributeViInt32 (instrSession, "", RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_LOGARITHMIC_COUNT, 
						                       count), 2, "Count");
	
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function Returns the results of the safe acquisition point measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the result type.
/// HIPAR result/Returns the results of the safe acquisition point measurement..
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSafeOperatingAcquisitionPointResults (ViSession instrSession,
                                                                            ViInt32 pointNumber,
                                                                            ViInt32 valueType,
                                                                            ViReal64 *result)
{
	ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, valueType, RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT, RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE), 3, "Value Type");

	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_SELECTED_POINT, pointNumber), 2, "Point Number");
	
	switch (valueType)
    {
        case RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_VIOLATION_CURRENT,
											   result), 4, "Result");
            break;                
        case RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_ACQUISITION_VIOLATION_VOLTAGE,
											   result), 4, "Result");
            break;
	
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value Type");
		break;
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;

}

/// HIFN  This function returns the voltage/current results of the total violation of SOA point measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the result type.
/// HIPAR result/Returns the voltage and the current of the total violation of SOA point measurement.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerSafeOperatingTotalPointResults (ViSession instrSession,
                                                                      ViInt32 pointNumber,
                                                                      ViInt32 valueType,
                                                                      ViReal64 *result)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, valueType, RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT, RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE), 3, "Value Type");

	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_SELECTED_POINT, pointNumber), 2, "Point Number");
	
	switch (valueType)
    {
        case RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VIOLATION_CURRENT,
											   result), 4, "Result");
            break;                
        case RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE:
            viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
	                                           RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VIOLATION_VOLTAGE,
											   result), 4, "Result");
            break;
	
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value Type");
		break;
    }

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function initiates measurement and returns returns the current or voltage data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the result type.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read waveform operation to complete.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array. The time is relative to the trigger event.
/// HIPAR xIncrement/Indicates the length of time between points in the WaveformArray.
ViStatus _VI_FUNC rsrtm20xx_ReadPowerSafeOperatingAreaTotalData (ViSession instrSession,
                                                                 ViInt32 pointNumber,
                                                                 ViInt32 valueType,
                                                                 ViInt32 waveformSize,
                                                                 ViInt32 maximumTime,
                                                                 ViReal64 waveformArray[],
                                                                 ViInt32 *actualPoints,
                                                                 ViReal64 *initialX,
                                                                 ViReal64 *xIncrement)
{
	ViStatus	error = VI_SUCCESS;

	ViInt32     old_tout = 0;

	checkErr (Rs_LockSession (instrSession, VI_NULL));
	checkErr (rsrtm20xx_GetOPCTimeout (instrSession, &old_tout));
	checkErr (rsrtm20xx_SetOPCTimeout (instrSession, maximumTime));
	
    checkErr (rsrtm20xx_InitiateAcquisition (instrSession, RSRTM20XX_VAL_INIT_SINGLE));
    checkErr (rsrtm20xx_FetchPowerSafeOperatingAreaTotalData (instrSession, pointNumber, valueType, waveformSize,
                                                                             waveformArray, actualPoints, initialX,
                                                                             xIncrement));
    
Error:
	rsrtm20xx_SetOPCTimeout (instrSession, old_tout);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the current or voltage data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the result type.
/// HIPAR waveformSize/Pass the number of elements in the Waveform Array parameter.
/// HIPAR waveformArray/Returns the waveform that the oscilloscope acquires.
/// HIPAR actualPoints/Indicates the number of points the function places in the Waveform Array parameter.
/// HIPAR initialX/Indicates the time of the first point in the Waveform Array. The time is relative to the trigger event.
/// HIPAR xIncrement/Indicates the length of time between points in the WaveformArray.
ViStatus _VI_FUNC rsrtm20xx_FetchPowerSafeOperatingAreaTotalData (ViSession instrSession,
                                                                  ViInt32 pointNumber,
                                                                  ViInt32 valueType,
                                                                  ViInt32 waveformSize,
                                                                  ViReal64 waveformArray[],
                                                                  ViInt32 *actualPoints,
                                                                  ViReal64 *initialX,
                                                                  ViReal64 *xIncrement)
{
	ViStatus	error = VI_SUCCESS;
	ViChar    buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, valueType, RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT, RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE), 3, "Value Type");

	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_SELECTED_POINT, pointNumber), 2, "Point Number");
	
	switch (valueType)
	{
		case RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT:
			
             viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_X_START, initialX), 7, "Initial X");
             viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_X_INCREMENT, xIncrement), 8, "X Increment");

		
		
		
             sprintf (buffer, "FORM REAL,32;POW:SOA:RES:TOT:VIOL%ld:CURR:DATA?", pointNumber);
             checkErr (rsrtm20xx_dataReadTrace (instrSession, buffer, waveformSize, waveformArray, actualPoints));
	
		break;
		
	    case RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE:
			viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_X_START, initialX), 7, "Initial X");
             viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_X_INCREMENT, xIncrement), 8, "X Increment");

		
		
		
             sprintf (buffer, "FORM REAL,32;POW:SOA:RES:TOT:VIOL%ld:VOLT:DATA?", pointNumber);
             checkErr (rsrtm20xx_dataReadTrace (instrSession, buffer, waveformSize, waveformArray, actualPoints));
			 
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value Type");
		break;
	}
Error:	
	if (p2data) free (p2data);
    viPrintf (instrSession, "FORM ASCII");
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns information of the current or voltage data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the result type.
/// HIPAR xStart/X Start in seconds.
/// HIPAR xStop/X Stop in seconds.
/// HIPAR recordLength/Record length of the waveform in Samples.
/// HIPAR valuesPerSample/Number of values per sample interval, usually 2.
ViStatus _VI_FUNC rsrtm20xx_FetchPowerSafeOperatingAreaTotalDataHeader (ViSession instrSession,
                                                                        ViInt32 pointNumber,
                                                                        ViInt32 valueType,
                                                                        ViReal64 *xStart,
                                                                        ViReal64 *xStop,
                                                                        ViInt32 *recordLength,
                                                                        ViInt32 *valuesPerSample)
{
	ViStatus	error = VI_SUCCESS;

    ViChar   header[RSRTM20XX_IO_BUFFER_SIZE];
    ViChar   *p2buf  = NULL;
    ViChar   *p2data  = NULL;
    ViUInt32 count = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, valueType, RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT, RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE), 3, "Value Type");
    
	switch (valueType)
	{
		case RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT:
			
             sprintf (header, "POW:SOA:RES:TOT:VIOL%ld:CURR:DATA:HEAD?\n", pointNumber);
             checkErr (viWrite (instrSession, (ViBuf) header, (ViUInt32) strlen (header), NULL));
             checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
             checkErr (rsrtm20xx_CheckStatus (instrSession));
    	break;
		
		case RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE:
			
			 sprintf (header, "POW:SOA:RES:TOT:VIOL%ld:VOLT:DATA:HEAD?\n", pointNumber);
             checkErr (viWrite (instrSession, (ViBuf) header, (ViUInt32) strlen (header), NULL));
             checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
             checkErr (rsrtm20xx_CheckStatus (instrSession));
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value Type");
		break;
	 }
	
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStart != NULL) *xStart = atof (p2buf);
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (xStop != NULL) *xStop = atof (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (recordLength != NULL) *recordLength = atol (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    if (valuesPerSample != NULL) *valuesPerSample = atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the conversion data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR pointNumber/Sets the point number.
/// HIPAR valueType/Sets the result type.
/// HIPAR xStart/Return the time of the first sample of the indicated waveform.
/// HIPAR xIncrement/Return the time difference between two adjacent samples of the indicated waveform.
/// HIPAR yStart/Return the voltage value for binary value 0 of the indicated waveform.
/// HIPAR yIncrement/Return the voltage value per bit of the indicated waveform.
/// HIPAR yResolution/Return the vertical bit resolution of the indicated waveform.
ViStatus _VI_FUNC rsrtm20xx_FetchPowerSafeOperatingAreaTotalConversionData (ViSession instrSession,
                                                                            ViInt32 pointNumber,
                                                                            ViInt32 valueType,
                                                                            ViReal64 *xStart,
                                                                            ViReal64 *xIncrement,
                                                                            ViReal64 *yStart,
                                                                            ViReal64 *yIncrement,
                                                                            ViInt32 *yResolution)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, valueType, RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT, RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE), 3, "Value Type");
	
	viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
	        RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_SELECTED_POINT, pointNumber), 2, "Point Number");
	
	switch (valueType)
	{
		case RSRTM20XX_VAL_POWER_SOA_TYPE_CURRENT:
			
		viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_X_START, xStart), 4, "X Start");
        viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_X_INCREMENT, xIncrement), 5, "X Increment");	
		viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_Y_START, yStart), 6, "Y Start");
        viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_Y_INCREMENT, yIncrement), 7, "Y Increment");
		viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_CURRENT_DATA_CONVERSION_Y_RESOLUTION, yResolution), 8, "Y Resolution");
        
		break;
		
		case RSRTM20XX_VAL_POWER_SOA_TYPE_VOLTAGE:
			
		viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_X_START, xStart), 4, "X Start");
        viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_X_INCREMENT, xIncrement), 5, "X Increment");	
		viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_Y_START, yStart), 6, "Y Start");
        viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_Y_INCREMENT, yIncrement), 7, "Y Increment");
		viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, VI_NULL,
                         RSRTM20XX_ATTR_POWER_SAFE_OPERATING_AREA_RESULT_TOTAL_VOLTAGE_DATA_CONVERSION_Y_RESOLUTION, yResolution), 8, "Y Resolution");
        
		break;
		
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value Type");
		break;
	 }
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function starts the power transient response measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_StartPowerTransientResponseMeasurement (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_START_POWER_TRANSIENT_RESPONSE_MEASUREMENT, NULL));
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function starts the power transient response measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR signalHigh/Sets the expected signal high voltage value.
/// HIPAR signalLow/Sets the expected signal low voltage value.
/// HIPAR startTime/Sets the start time.
/// HIPAR stopTime/Sets the stop time.
ViStatus _VI_FUNC rsrtm20xx_TransientResponseSettings (ViSession instrSession,
                                                       ViReal64 signalHigh,
                                                       ViReal64 signalLow,
                                                       ViReal64 startTime,
                                                       ViReal64 stopTime)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_SIGNAL_HIGH, signalHigh), 2, "Signal High");    
    
	viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_SIGNAL_LOW, signalLow), 3, "Signal Low");    
	
	viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_START_TIME, startTime), 4, "Start Time");    
	
	viCheckParm(rsrtm20xx_SetAttributeViReal64 (instrSession, NULL,
        RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_STOP_TIME, stopTime), 5, "Stop Time");    
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function adds the result to the report list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_TransientResponseAddToReport (ViSession instrSession)

{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_ADD_REPORT, NULL));
	
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the results of the transient response measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the result type.
/// HIPAR result/Returns the result.
ViStatus _VI_FUNC rsrtm20xx_QueryPowerTransientResponseResult (ViSession instrSession,
                                                               ViInt32 type,
                                                               ViReal64 *result)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, type, RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_OVERSHOOT, 
											RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_SETTLING), 2, "Type");
	switch (type)
	{
		
		
		case RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_OVERSHOOT:
			 viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
					        					RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_OVERSHOOT, 
												result), 3, "Result");
			 break;
		case RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_RISE_TIME:
			 viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
					        					RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_RISE_TIME, 
												result), 3, "Result");
			 break;
		case RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_DELAY_TIME:
					viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
					        					RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_DELAY_TIME, 
												result), 3, "Result");
			 break;
		case RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_PEAK_TIME:
					viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
					        					RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_PEAK_TIME, 
												result), 3, "Result");
			break;
		case RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_PEAK_VALUE:
					viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
					        					RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_PEAK_VALUE, 
												result), 3, "Result");
			break;
		case RSRTM20XX_VAL_POWER_TRANSIENT_MEAS_SETTLING:
					viCheckParm(rsrtm20xx_GetAttributeViReal64(instrSession, "",
					        					RSRTM20XX_ATTR_POWER_TRANSIENT_RESPONSE_SETTLING_TIME, 
												result), 3, "Result");
			break;
			default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Type");
			break;
	}
	
Error:
(void) Rs_UnlockSession (instrSession, VI_NULL);
	return error;
}

/// HIFN  This function enables the spectrum analysis.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_ANALYSIS_ENABLE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SPECtrum[:STATe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control enables the spectrum analysis.
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisState(
	ViSession	instrSession,
	ViBoolean	state
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, "",
                                                  RSRTM20XX_ATTR_SPECTRUM_ANALYSIS_ENABLE,
                                                  state), 2, "State");
    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function selects the source for the spectrum analysis diagrams.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_ANALYSIS_SOURCE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SPECtrum:SOURce
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/Selects the source for the spectrum analysis diagrams.
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisSource(
	ViSession	instrSession,
	ViInt32	source
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, "",
                                                RSRTM20XX_ATTR_SPECTRUM_ANALYSIS_SOURCE,
                                                source), 2, "Source");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the spectrum analysis frequency.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_START
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_STOP
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_CENTER
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_SPAN
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SPECtrum:FREQuency:STARt
/// HIFN     SPECtrum:FREQuency:STOP
/// HIFN     SPECtrum:FREQuency:CENTer
/// HIFN     SPECtrum:FREQuency:SPAN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR start/Defines the start frequency of the displayed frequency domain at the 
/// HIPAR start/left display edge: Center - Span/2
/// HIPAR stop/Defines the stop frequency of the displayed frequency domain at the 
/// HIPAR stop/right display edge: Center + Span/2
/// HIPAR center/Defines the position of the displayed frequency domain, which is 
/// HIPAR center/(Center - Span/2) to (Center + Span/2). The width of the domain is 
/// HIPAR center/defined using the command SPECtrum:FREQuency:SPAN.
/// HIPAR span/The span is specified in Hertz and defines the width of the displayed 
/// HIPAR span/frequency range, which is (Center - Span/2) to (Center + Span/2).
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisFrequency(
	ViSession	instrSession,
	ViReal64	start,
	ViReal64	stop,
	ViReal64	center,
	ViReal64	span
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_SetAttributeViReal64 (instrSession, "",
                                                 RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_START,
                                                 start), 2, "Start");
	viCheckParm (rsrtm20xx_SetAttributeViReal64 (instrSession, "",
                                                 RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_STOP,
                                                 stop), 3, "Stop");
	viCheckParm (rsrtm20xx_SetAttributeViReal64 (instrSession, "",
                                                 RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_CENTER,
                                                 center), 4, "Center");
	viCheckParm (rsrtm20xx_SetAttributeViReal64 (instrSession, "",
                                                 RSRTM20XX_ATTR_SPECTRUM_FREQUENCY_SPAN,
                                                 span), 5, "Span");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function enables the spectrum analysis.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_ENABLE
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_MINIMUM_ENABLE
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_MAXIMUM_ENABLE
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_AVERAGE_ENABLE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SPECtrum:WAVeform:SPECtrum[:ENABle]
/// HIFN     SPECtrum:WAVeform:MINimum[:ENABle]
/// HIFN     SPECtrum:WAVeform:MAXimum[:ENABle]
/// HIFN     SPECtrum:WAVeform:AVERage[:ENABle]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR spectrum/Enables or diables the indicated waveform.
/// HIPAR minHold/Enables or diables the indicated waveform.
/// HIPAR maxHold/Enables or diables the indicated waveform.
/// HIPAR average/Enables or diables the indicated waveform.
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisWaveform(
	ViSession	instrSession,
	ViBoolean	spectrum,
	ViBoolean	minHold,
	ViBoolean	maxHold,
	ViBoolean	average
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, "",
                                                  RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_ENABLE,
                                                  spectrum), 2, "Spectrum");
	viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, "",
                                                  RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_MINIMUM_ENABLE,
                                                  minHold), 3, "Min Hold");
	viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, "",
                                                  RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_MAXIMUM_ENABLE,
                                                  maxHold), 4, "Max Hold");
	viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, "",
                                                  RSRTM20XX_ATTR_SPECTRUM_WAVEFORM_AVERAGE_ENABLE,
                                                  average), 5, "Average");
    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the spectrum analysis time domain.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_TIME_DOMAIN_POSITION
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_TIME_DOMAIN_RANGE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SPECtrum:TIME:POSition
/// HIFN     SPECtrum:TIME:RANGe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR position/Sets the time position of the analyzed time range.
/// HIPAR range/Sets the time range for the time domain diagram.
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisTimeDomain(
	ViSession	instrSession,
	ViReal64	position,
	ViReal64	range
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_SetAttributeViReal64 (instrSession, "",
                                                 RSRTM20XX_ATTR_SPECTRUM_TIME_DOMAIN_POSITION,
                                                 position), 2, "Position");
	viCheckParm (rsrtm20xx_SetAttributeViReal64 (instrSession, "",
                                                 RSRTM20XX_ATTR_SPECTRUM_TIME_DOMAIN_RANGE,
                                                 range), 3, "Range");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the spectrum analysis display.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_DISPLAY_COLOR_SCHEME
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_DISPLAY_MAGNITUDE_MODE
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_DISPLAY_MINIMUM_LEVEL
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_DISPLAY_MAXIMUM_LEVEL
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SPECtrum:DIAGram:COLor:SCHeme:FDOMain
/// HIFN     SPECtrum:DIAGram:COLor:MAGNitude:MODE
/// HIFN     SPECtrum:DIAGram:COLor:MINimum[:LEVel]
/// HIFN     SPECtrum:DIAGram:COLor:MAXimum[:LEVel]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR colorScheme/Sets the color scale for the display of the waveform in the frequency 
/// HIPAR colorScheme/domain diagram.
/// HIPAR magnitudeMode/Enables the magnitude coloring of the waveform.
/// HIPAR minimum/Sets the level used as a minimum of the color scale selected with 
/// HIPAR minimum/SPECtrum:DIAGram:COLor:SCHeme:FDOMain/SPECtrum:DIAGram:COLor:SCHeme:SPECtrogramm. 
/// HIPAR minimum/All level values lower than the minimum will be displayed with the 
/// HIPAR minimum/minimum color.
/// HIPAR maximum/Sets the level used as a maximum of the color scale selected with 
/// HIPAR maximum/SPECtrum:DIAGram:COLor:SCHeme:FDOMain/SPECtrum:DIAGram:COLor:SCHeme:SPECtrogramm. 
/// HIPAR maximum/All level values lower than the minimum will be displayed with the 
/// HIPAR maximum/minimum color.
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisDisplay(
	ViSession	instrSession,
	ViInt32	colorScheme,
	ViBoolean	magnitudeMode,
	ViReal64	minimum,
	ViReal64	maximum
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, "",
                                                RSRTM20XX_ATTR_SPECTRUM_DISPLAY_COLOR_SCHEME,
                                                colorScheme), 2, "Color Scheme");
	viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, "",
                                                  RSRTM20XX_ATTR_SPECTRUM_DISPLAY_MAGNITUDE_MODE,
                                                  magnitudeMode), 3, "Magnitude Mode");
    viCheckParm (rsrtm20xx_SetAttributeViReal64 (instrSession, "",
                                                 RSRTM20XX_ATTR_SPECTRUM_DISPLAY_MINIMUM_LEVEL,
                                                 minimum), 4, "Minimum");
	viCheckParm (rsrtm20xx_SetAttributeViReal64 (instrSession, "",
                                                 RSRTM20XX_ATTR_SPECTRUM_DISPLAY_MAXIMUM_LEVEL,
                                                 maximum), 5, "Maximum");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function Enables the usage of markers.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_MARKER_ENABLE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SPECtrum:MARKer[:ENABle]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables the usage of markers.
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisMarkerState(
	ViSession	instrSession,
	ViBoolean	state
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, "",
                                                  RSRTM20XX_ATTR_SPECTRUM_MARKER_ENABLE,
                                                  state), 2, "State");
    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function sets the source for the marker search function.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_MARKER_SOURCE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SPECtrum:MARKer:SOURce
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR source/Sets the source for the marker search function.
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisMarkerSource(
	ViSession	instrSession,
	ViInt32	source
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, "",
                                                RSRTM20XX_ATTR_SPECTRUM_MARKER_SOURCE,
                                                source), 2, "Source");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function sets the mode for the selection of the reference peak. 
/// HIFN  The peak with the highest level within the selected settings is set as 
/// HIFN  the reference marker.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_SPECTRUM_MARKER_REFERENCE_MODE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SPECtrum:MARKer:REFerence:SETup:MODE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR reference/Sets the mode for the selection of the reference peak. The peak with 
/// HIPAR reference/the highest level within the selected settings is set as the reference 
/// HIPAR reference/marker.
ViStatus _VI_FUNC rsrtm20xx_ConfigureSpectrumAnalysisMarkerReference(
	ViSession	instrSession,
	ViInt32	reference
)
{
    ViStatus  error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, "",
                                                RSRTM20XX_ATTR_SPECTRUM_MARKER_REFERENCE_MODE,
                                                reference), 2, "Reference");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function enables and disables all configured voltmeter 
/// HIFN  measurements.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_DVM_ENABLE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DVM<m>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurementPlace/Sets the measurement place.
/// HIPAR state/Enables and disables all configured voltmeter measurements.
ViStatus _VI_FUNC rsrtm20xx_ConfigureDVMState(
	ViSession	instrSession,
	ViInt32	measurementPlace,
	ViBoolean	state
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm(rsrtm20xx_invalidViInt32Range (instrSession, measurementPlace, 1, 4), 2, "Measurement Place");

    sprintf (rep_cap, "DVM%ld", measurementPlace);  

    viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, rep_cap,
                                                  RSRTM20XX_ATTR_DVM_ENABLE,
                                                  state), 3, "State");
    
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function configures the Digital Voltmeter.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_DVM_SOURCE
/// HIFN     RSRTM20XX_ATTR_DVM_TYPE
/// HIFN     RSRTM20XX_ATTR_DVM_POSITION
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DVM<m>:SOURce
/// HIFN     DVM<m>:TYPE
/// HIFN     DVM<m>:POSition
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurementPlace/Sets the measurement place.
/// HIPAR source/Sets the measurement source for the indicated DVM measurement.
/// HIPAR type/Sets the measurement type for the indicated DVM measurement.
/// HIPAR position/Sets the corner of the screen in which the measurement results are 
/// HIPAR position/displayed.
ViStatus _VI_FUNC rsrtm20xx_ConfigureDVM(
	ViSession	instrSession,
	ViInt32	measurementPlace,
	ViInt32	source,
	ViInt32	type,
	ViInt32	position
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_invalidViInt32Range (instrSession, measurementPlace, 1, 4), 2, "Measurement Place");

    sprintf (rep_cap, "DVM%ld", measurementPlace);

    viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_DVM_SOURCE,
                                                source), 3, "Source");
	viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_DVM_TYPE,
                                                type), 4, "Type");
	viCheckParm (rsrtm20xx_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSRTM20XX_ATTR_DVM_POSITION,
                                                position), 5, "Position");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function returns the current value of the indicated measurement.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_DVM_RESULT
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     DVM<m>:RESult[:ACTual]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR measurementPlace/Sets the measurement place.
/// HIPAR result/Returns the current value of the indicated measurement.
ViStatus _VI_FUNC rsrtm20xx_QueryDVMResult(
	ViSession	instrSession,
	ViInt32	measurementPlace,
	ViReal64*	result
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_invalidViInt32Range (instrSession, measurementPlace, 1, 4), 2, "Measurement Place");

    sprintf (rep_cap, "DVM%ld", measurementPlace);  

    viCheckParm (rsrtm20xx_GetAttributeViReal64 (instrSession, rep_cap,
                                                 RSRTM20XX_ATTR_DVM_RESULT,
                                                 result), 3, "Result");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}


/// HIFN  This function enables or disables the trigger counter measurements.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSRTM20XX_ATTR_COUNTER_ENABLE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     TCOunter<t>:ENAB
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR counter/Sets the counter.
/// HIPAR state/Enables or disables the trigger counter measurements.
ViStatus _VI_FUNC rsrtm20xx_ConfigureCounterState(
	ViSession	instrSession,
	ViInt32	counter,
	ViBoolean	state
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rep_cap[15] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsrtm20xx_invalidViInt32Range (instrSession, counter, 1, 2), 2, "Counter");

    sprintf (rep_cap, "Counter%ld", counter);  

    viCheckParm (rsrtm20xx_SetAttributeViBoolean (instrSession, rep_cap,
                                                  RSRTM20XX_ATTR_COUNTER_ENABLE,
                                                  state), 3, "State");
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
ViStatus _VI_FUNC rsrtm20xx_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rsrtm20xx_DefaultInstrSetup (instrSession));
    checkErr( rsrtm20xx_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);

    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rsrtm20xx_ConfigureErrorChecking (ViSession instrSession,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     p2value         = 0;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    sessionProperties -> optionChecking = optionChecking;
    p2value = (ViInt32) statusChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, &p2value));
    p2value = (ViInt32) rangeChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_RANGE_CHECK, 0, &p2value));
	
Error:
    return error;
}


/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsrtm20xx_self_test (ViSession instrSession, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    viCheckErr (rsrtm20xx_GetAttributeViInt32 (instrSession, NULL, RSRTM20XX_ATTR_SELF_TEST_QUERY, &result));
	
	*testResult = (ViInt16) result;
	
    if (*testResult == 0)
        strcpy (testMessage, "Self-Test Passed.");
    else
        strcpy (testMessage, "Self-Test Failed.");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET /// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsrtm20xx_revision_query (ViSession instrSession, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSRTM20XX_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSRTM20XX_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsrtm20xx_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsrtm20xx_error_query (ViSession instrSession, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViInt32     oldCode = 0;
    ViChar      oldMessage[256] = "";
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

   
	if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
    {
        p2buf = (char *)malloc (1);

        *p2buf='\0';
            
        do 
        {
            checkErr (viWrite (instrSession, (ViBuf) "SYST:ERR?\n", 10, NULL));
			checkErr (viRead (instrSession, (ViBuf)buffer, RSRTM20XX_IO_BUFFER_SIZE, NULL));
            if  (sscanf (buffer, "%ld,%*[^0-9a-zA-Z]%256[^\"]", &oldCode, oldMessage) != 2)
            {
                viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);
            }
            else
            {
                if (oldCode != 0)
                {
                    *errCode = oldCode;
                    
                    p2buf = (char *)realloc (p2buf, strlen(p2buf) + strlen (oldMessage) + 2);
        
                    strcat (p2buf, oldMessage);
        
                    strcat (p2buf, "\n");
                    
                    i++;
                }
                else if (i == 0)
                {
                    *errCode = oldCode;
    
                    strncpy (errMessage, oldMessage, 1024);
                }
            }

        } while (oldCode != 0);
        
        if (i > 0)
        {
            p2buf[strlen (p2buf) - 1] = '\0'; 
            
            strncpy (errMessage, p2buf, 1024);
        }
    }
    
    else
    {
        /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
    }

Error:
    if (p2buf) free (p2buf);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/// HIFN Queries the error/event queue for all unread items and removes them from the queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR bufferSize/Sets the size of Errors string.
/// HIPAR errors/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsrtm20xx_ErrorList (ViSession instrSession,
                                   ViInt32 bufferSize, ViChar errors[])
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsrtm20xx_GetAttributeViString (instrSession, NULL,
                                                 RSRTM20XX_ATTR_SYST_ERR_LIST, bufferSize,
                                                 errors), 3, "Errors"); 
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsrtm20xx_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSRTM20XX_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function resets all STATUS:QUESTIONALBLE registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsrtm20xx_ResetStatusRegisters (ViSession instrSession)
{
	ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, NULL, RSRTM20XX_ATTR_RESET_STATUS_REGISTER, NULL)); 
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function sets the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerType/This control selects type of the registers that will be configured.
/// HIPAR statusRegister/This control selects the registers that will be configured.
/// HIPAR registerPart/This control selects the registers that will be configured.
/// HIPAR registerValue/Value of selected register.
ViStatus _VI_FUNC rsrtm20xx_SetStatusRegister (ViSession instrSession,
                                             ViInt32 registerType,
                                             ViInt32 statusRegister,
                                             ViInt32 registerPart,
                                             ViInt32 registerValue)
{
    ViStatus error = VI_SUCCESS;
	switch (registerType)
	{
		case RSRTM20XX_VAL_REG_OPER:
			switch (registerPart)
			{
				case RSRTM20XX_VAL_REG_ENABLE:
					viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
					        					RSRTM20XX_ATTR_OPERATION_ENABLE, 
												registerValue), 5, "Register Value");
					break;
						case RSRTM20XX_VAL_REG_NTR:
					viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
					        					RSRTM20XX_ATTR_OPERATION_N_TRANSITION, 
												registerValue), 5, "Register Value");
					break;
				case RSRTM20XX_VAL_REG_PTR:
					viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
					        					RSRTM20XX_ATTR_OPERATION_P_TRANSITION, 
												registerValue), 5, "Register Value");
					break;
				default:
					viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
					break;
			}
			break;
		case RSRTM20XX_VAL_REG_QUES:
			switch (statusRegister)
			{
				case RSRTM20XX_VAL_REG_NONE:
					switch (registerPart)
					{
						case RSRTM20XX_VAL_REG_ENABLE:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_ENABLE, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_NTR:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_N_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_PTR:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_P_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						default:
							viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
							break;
					}
					break;
				case RSRTM20XX_VAL_REG_COV:
					switch (registerPart)
					{
						case RSRTM20XX_VAL_REG_ENABLE:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_ENABLE, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_NTR:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_N_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_PTR:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_P_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						default:
							viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
							break;
					}
					break;
				case RSRTM20XX_VAL_REG_LIM:
					switch (registerPart)
					{
						case RSRTM20XX_VAL_REG_ENABLE:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_ENABLE, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_NTR:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_N_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_PTR:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_P_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						default:
							viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
							break;
					}
					break;
				case RSRTM20XX_VAL_REG_MASK:
					switch (registerPart)
					{
						
						case RSRTM20XX_VAL_REG_ENABLE:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_MASK_ENABLE, 
														registerValue), 5, "Register Value");
							break;
						
						case RSRTM20XX_VAL_REG_NTR:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_MASK_N_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_PTR:
							viCheckParm(rsrtm20xx_SetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_MASK_P_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						default:
							viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
							break;
					}
					break;
				default:
					viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Status Register");
					break;
			}
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Register Type");
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function queries the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerType/This control selects type of the registers that will be queried.
/// HIPAR statusRegister/This control selects the registers that will be queried.
/// HIPAR registerPart/This control selects the registers that will be queried.
/// HIPAR registerValue/Returns value of selected register.
ViStatus _VI_FUNC rsrtm20xx_GetStatusRegister (ViSession instrSession,
                                                ViInt32 registerType,
                                                ViInt32 statusRegister,
                                                ViInt32 registerPart,
                                                ViInt32 *registerValue)
{
    ViStatus error = VI_SUCCESS;
	switch (registerType)
	{
		case RSRTM20XX_VAL_REG_OPER:
			switch (registerPart)
			{
				case RSRTM20XX_VAL_REG_COND:
					viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
					        					RSRTM20XX_ATTR_OPERATION_CONDITION, 
												registerValue), 5, "Register Value");
					break;
				case RSRTM20XX_VAL_REG_ENABLE:
					viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
					        					RSRTM20XX_ATTR_OPERATION_ENABLE, 
												registerValue), 5, "Register Value");
					break;
				case RSRTM20XX_VAL_REG_EVENT:
					viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
					        					RSRTM20XX_ATTR_OPERATION_EVENT, 
												registerValue), 5, "Register Value");
					break;
				case RSRTM20XX_VAL_REG_NTR:
					viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
					        					RSRTM20XX_ATTR_OPERATION_N_TRANSITION, 
												registerValue), 5, "Register Value");
					break;
				case RSRTM20XX_VAL_REG_PTR:
					viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
					        					RSRTM20XX_ATTR_OPERATION_P_TRANSITION, 
												registerValue), 5, "Register Value");
					break;
				default:
					viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
					break;
			}
			break;
		case RSRTM20XX_VAL_REG_QUES:
			switch (statusRegister)
			{
				case RSRTM20XX_VAL_REG_NONE:
					switch (registerPart)
					{
						case RSRTM20XX_VAL_REG_COND:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_CONDITION, 
														registerValue), 5, "Register Value"); 
							break;
						case RSRTM20XX_VAL_REG_ENABLE:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_ENABLE, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_EVENT:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_EVENT, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_NTR:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_N_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_PTR:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_P_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						default:
							viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
							break;
					}
					break;
				case RSRTM20XX_VAL_REG_COV:
					switch (registerPart)
					{
						case RSRTM20XX_VAL_REG_COND:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_CONDITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_ENABLE:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_ENABLE, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_EVENT:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_EVENT, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_NTR:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_N_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_PTR:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_CHANNEL_OVERLOAD_P_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						default:
							viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
							break;
					}
					break;
				case RSRTM20XX_VAL_REG_LIM:
					switch (registerPart)
					{
						case RSRTM20XX_VAL_REG_COND:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_CONDITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_ENABLE:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_ENABLE, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_EVENT:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_EVENT, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_NTR:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_N_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_PTR:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_LIMIT_P_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						default:
							viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
							break;
					}
					break;
				case RSRTM20XX_VAL_REG_MASK:
					switch (registerPart)
					{
						case RSRTM20XX_VAL_REG_COND:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_MASK_CONDITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_ENABLE:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_MASK_ENABLE, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_EVENT:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_MASK_EVENT, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_NTR:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_MASK_N_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						case RSRTM20XX_VAL_REG_PTR:
							viCheckParm(rsrtm20xx_GetAttributeViInt32(instrSession, "",
							        					RSRTM20XX_ATTR_QUESTIONABLE_MASK_P_TRANSITION, 
														registerValue), 5, "Register Value");
							break;
						default:
							viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Register Part");
							break;
					}
					break;
				default:
					viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Status Register");
					break;
			}
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Register Type");
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rsrtm20xx_SetVISATimeout(ViSession instrSession,
                                  ViUInt32 VISATimeout)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, VISATimeout ));
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rsrtm20xx_GetVISATimeout(ViSession instrSession,
                                  ViUInt32 *VISATimeout)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, NULL));

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, VISATimeout ));
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function executes an internal self-alignment of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_SelfAlingment(
	ViSession	instrSession,
	ViInt32 	maximumTime
)
{
	ViStatus    error = VI_SUCCESS;
	ViInt32 	oldTimeout = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rsrtm20xx_GetOPCTimeout(instrSession, &oldTimeout));
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, maximumTime));

	viCheckErr (rsrtm20xx_SetAttributeViString (instrSession, "",
                                	RSRTM20XX_ATTR_SELF_ALIGNMENT, NULL));
	
	checkErr (rsrtm20xx_SetOPCTimeout(instrSession, oldTimeout));
    
Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;     
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_ClearStatus (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rsrtm20xx_WriteInstrData (instrSession, "*CLS\n"));
	
	checkErr( rsrtm20xx_CheckStatus (instrSession));

Error:
	(void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rsrtm20xx_IDQueryResponse (ViSession instrSession,
                                             ViChar IDQueryResponse[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rsrtm20xx_QueryViString (instrSession, "*IDN?", 256, IDQueryResponse));

Error:
	(void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrtm20xx_ProcessAllPreviousCommands (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rsrtm20xx_WriteInstrData (instrSession, "*WAI\n"));
	
Error:
	(void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rsrtm20xx_QueryOPC (ViSession instrSession, ViInt32 *OPC)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (rsrtm20xx_QueryViInt32 (instrSession, "*OPC?\n", OPC));

Error:
	(void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}


/// HIFN This function switches ON/OFF instrument status checking of the 
/// HIFN Tinstrument 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR statusChecking/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsrtm20xx_setCheckStatus (ViSession instrSession,
                               ViBoolean statusChecking)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     p2value         = 0;
    
    p2value = (ViInt32) statusChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, &p2value));

Error:
    return error;
}




/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsrtm20xx_GetError (ViSession instrSession, 
                                     ViStatus *errorCode, 
                                     ViInt32 bufferSize,
                                     ViChar description[])  
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }

    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsrtm20xx_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsrtm20xx_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
ViStatus _VI_FUNC rsrtm20xx_ClearError (ViSession instrSession)                                                        
{                                                                                                           
    return Rs_ClearErrorInfo (instrSession);                                                                             
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsrtm20xx_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsrtm20xx_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsrtm20xx_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsrtm20xx_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsrtm20xx_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsrtm20xx_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsrtm20xx_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsrtm20xx_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsrtm20xx_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsrtm20xx_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/// HIFN Sends a command to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for an instrument operation that can
/// HIFN take a long time to complete, e.g. self-alignment. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsrtm20xx_WriteCommandWithOPCSync (ViSession instrSession,
                                                    ViString writeBuffer,
                                                    ViUInt32 timeout)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
		
	checkErr( Rs_LockSession (instrSession, NULL)); 

	viCheckParm (strlen(writeBuffer) == 0, 2, "Write Buffer");
    
    viCheckErr (Rs_ClearBeforeRead (instrSession));
	
	sprintf (buffer, "%s;*OPC\n", writeBuffer);
	
	viCheckErr (Rs_WriteInstrData (instrSession, buffer));
	
	viCheckErr (rsrtm20xx_WaitForOPC (instrSession, timeout));
	
	checkErr (rsrtm20xx_CheckStatus (instrSession));
	
Error:
	(void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN Sends a query to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for a response that can take a long
/// HIFN time to complete e.g. self-alignment, calibration or self-test. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR bufferSize/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data
/// HIPAR readBuffer/that was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument
/// HIPAR numBytesRead/and stored in the Read Buffer.
ViStatus _VI_FUNC rsrtm20xx_QueryWithOPCSync (ViSession instrSession,
                                             ViString writeBuffer,
                                             ViUInt32 timeout,
                                             ViInt32 bufferSize,
                                             ViChar readBuffer[],
                                             ViUInt32 *numBytesRead)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      *pbuffer = NULL;
    ViUInt32    bytesRead;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
		
	checkErr( Rs_LockSession (instrSession, NULL)); 

	viCheckParm (strlen(writeBuffer) == 0, 2, "Write Buffer");
    viCheckParm (readBuffer == NULL, 5, "Read Buffer");

    viCheckErr (Rs_ClearBeforeRead (instrSession));
	
	sprintf (buffer, "%s;*OPC\n", writeBuffer);
	
	viCheckErr (Rs_WriteInstrData (instrSession, buffer));
	
	viCheckErr (rsrtm20xx_WaitForOPC (instrSession, timeout));
	
	viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(readBuffer, pbuffer, (size_t) (bufferSize-1));
        readBuffer[bufferSize-1]='\0';
    } 
    else
    {
        strncpy(readBuffer, pbuffer, (size_t) bytesRead); 
        readBuffer[bytesRead] = '\0';   
    }
	
	*numBytesRead = bytesRead;
    
	checkErr (rsrtm20xx_CheckStatus (instrSession));
	
Error:
	(void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function provides mass storage capabilities for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR source/This control specifies the source of the operation to be performed. If 
/// HIPAR source/the operation requires a single parameter, this control specifies that 
/// HIPAR source/parameter.
/// HIPAR destination/This control specifies the destination of the operation to be 
/// HIPAR destination/performed.
ViStatus _VI_FUNC rsrtm20xx_FileManagerOperations(
    ViSession   instrSession,
    ViInt32 operation,
    ViString    source,
    ViString    destination
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch (operation)
    {
        case RSRTM20XX_VAL_FILE_COPY:
            sprintf (buffer, "MMEM:COPY '%s','%s'\n", source, destination);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rsrtm20xx_CheckStatus (instrSession));
            break;
        case RSRTM20XX_VAL_FILE_MOVE:
            sprintf (buffer, "MMEM:MOVE '%s','%s'\n", source, destination);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rsrtm20xx_CheckStatus (instrSession));
            break;
        case RSRTM20XX_VAL_FILE_DELETE:
            viCheckParm (rsrtm20xx_SetAttributeViString(instrSession, "",
                RSRTM20XX_ATTR_DELETE_FILE, source), 3, "Source");
            break;
        case RSRTM20XX_VAL_FILE_NEW_DIR:
            viCheckParm (rsrtm20xx_SetAttributeViString(instrSession, "",
                RSRTM20XX_ATTR_CREATE_DIRECTORY, source), 3, "Source");
            break;
        case RSRTM20XX_VAL_FILE_CDIR:
            viCheckParm (rsrtm20xx_SetAttributeViString(instrSession, "",
                RSRTM20XX_ATTR_CURRENT_DIRECTORY, source), 3, "Source");
            break;
        case RSRTM20XX_VAL_FILE_RDIR:
            viCheckParm (rsrtm20xx_SetAttributeViString(instrSession, "",
                RSRTM20XX_ATTR_REMOVE_DIRECTORY, source), 3, "Source");
            break;
        case RSRTM20XX_VAL_FILE_CDRIVE:
            viCheckParm (rsrtm20xx_SetAttributeViString(instrSession, "",
                RSRTM20XX_ATTR_CHANGE_DRIVE, source), 3, "Source");
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the number of files and subdirectories of the current or of a 
/// HIFN  specified directory. The number includes the parent directory strings 
/// HIFN  "." and ".." so that it corresponds to the number of strings returned 
/// HIFN  by the rsrtm20xx_FileDirectoryContent function after the initial numeric 
/// HIFN  parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR numberofFilesAndFolders/Returns the number of files and subdirectories of the current or of a 
/// HIPAR numberofFilesAndFolders/specified directory.
ViStatus _VI_FUNC rsrtm20xx_FileCount(
    ViSession   instrSession,
    ViString    directory,
    ViInt32*    numberofFilesAndFolders
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar *p2buf = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (numberofFilesAndFolders);
    
    p2buf = buffer + sprintf (buffer, "MMEM:CAT:LENG?");
    
    if ((directory != NULL) && (strlen (directory) > 0))
        p2buf += sprintf (p2buf, " '%s'", directory);
    
    strcat (buffer, "\n");
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (viScanf (instrSession, "%ld", numberofFilesAndFolders));
    checkErr (rsrtm20xx_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function reads the indicated directory. According to DOS 
/// HIFN  convention, wild card characters can be entered in order to query eg. a 
/// HIFN  list of all file of a certain type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
/// HIPAR output/This indicator returns the files in the indicated directory as comma 
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus _VI_FUNC rsrtm20xx_FileDirectoryContent(
    ViSession   instrSession,
    ViString    directory,
    ViInt32 bufferSize,
    ViChar    output[]
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar   *p2buf = NULL;
    ViUInt32  count = 0;
    ViUInt32  length = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (output);
    
    p2buf = buffer + sprintf (buffer, "MMEM:CAT?");
    
    if ((directory != NULL) && (strlen (directory) > 0))
        p2buf += sprintf (p2buf, " '%s'", directory);
    
    strcat (buffer, "\n");
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    p2buf = NULL;
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2buf, &count));
    
    length = ((ViInt32)count > bufferSize) ? bufferSize : count;
    memcpy (output, p2buf, length);
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));

Error:
    if (p2buf != NULL) free (p2buf);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the number of subdirectories of the current or of a specified 
/// HIFN  directory. The number includes the parent directory strings "." and 
/// HIFN  ".." so that it corresponds to the number of strings returned by the 
/// HIFN  rsrtm20xx_FileSubdirectories.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR numberofSubdirectories/Returns the number of files and subdirectories of the current or of a 
/// HIPAR numberofSubdirectories/specified directory.
ViStatus _VI_FUNC rsrtm20xx_FileSubdirectoriesCount(
    ViSession   instrSession,
    ViString    directory,
    ViInt32*    numberofSubdirectories
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar *p2buf = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (numberofSubdirectories);
    
    p2buf = buffer + sprintf (buffer, "MMEM:DCAT:LENG?");
    
    if ((directory != NULL) && (strlen (directory) > 0))
        p2buf += sprintf (p2buf, " '%s'", directory);
    
    strcat (buffer, "\n");
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (viScanf (instrSession, "%ld", numberofSubdirectories));
    checkErr (rsrtm20xx_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the subdirectories of the current or of a specified directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be 
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
/// HIPAR output/This indicator returns the files in the indicated directory as comma 
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus _VI_FUNC rsrtm20xx_FileSubdirectories(
    ViSession   instrSession,
    ViString    directory,
    ViInt32 bufferSize,
    ViChar    output[]
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViChar   *p2buf = NULL;
    ViUInt32  count = 0;
    ViUInt32  length = 0;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckAlloc (output);
    
    p2buf = buffer + sprintf (buffer, "MMEM:DCAT?");
    
    if ((directory != NULL) && (strlen (directory) > 0))
        p2buf += sprintf (p2buf, " '%s'", directory);
    
    strcat (buffer, "\n");
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    p2buf = NULL;
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2buf, &count));
    
    length = ((ViInt32)count > bufferSize) ? bufferSize : count;
    memcpy (output, p2buf, length);
    
    checkErr (rsrtm20xx_CheckStatus (instrSession));

Error:
    if (p2buf != NULL) free (p2buf);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns a list of the drives of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/Returns a list of the drives of the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/Output parameter.  
ViStatus _VI_FUNC rsrtm20xx_FileDrives(
    ViSession   instrSession,
    ViInt32   bufferSize,
    ViChar    output[]
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm(rsrtm20xx_GetAttributeViString(instrSession, "",
        RSRTM20XX_ATTR_DRIVES, bufferSize, output), 2, "Drives");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function is used to read data from the instrument and write it to 
/// HIFN  a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument to 
/// HIPAR source/control computer.
/// HIPAR destination/This control defines destination file to which the data transfer from 
/// HIPAR destination/instrument to control computer applies.
ViStatus _VI_FUNC rsrtm20xx_ReadToFileFromInstrument(
    ViSession   instrSession,
    ViString    source,
    ViString    destination
)
{
    ViStatus    error = VI_SUCCESS;
    //ViInt32     count = 0;
    ViUInt32    ret_count = 0;
    ViUInt64    length = 0;
    ViChar      buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";
    ViUInt16 intfc = VI_INTF_GPIB;     
	ViChar      *pbuffer = NULL;
	int i;
    
    checkErr( Rs_LockSession (instrSession, NULL));
    
	viCheckParm (!source, 2, "Source");
	viCheckParm (!destination, 3, "Destination");    
    
	checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));
    switch (intfc)
    {
        case VI_INTF_ASRL: /* Serial intfc. */
            viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        break;
    }
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
    viCheckErr (viPrintf(instrSession, ":MMEMory:DATA? '%s'\n", source));
    checkErr (viRead (instrSession, (ViBuf) buffer, 1, NULL)); 

	
	if (buffer[0] == '#')
	{   /* Binary data */
		/* Disable termination character */
		checkErr(viSetAttribute(instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

		/* Parse header */
		checkErr(viRead(instrSession, (ViBuf)buffer, 1, NULL));

		buffer[1] = '\0';
		if (buffer[0] == '(')
		{
			for (i = 0; i < 20; i++)
			{
				checkErr(viRead(instrSession, (ViBuf)buffer + i, 1, NULL));

				if (buffer[i] == ')')
				{
					buffer[i] = '\0';
					sscanf(buffer, "%lld", &length);
					break;
				}
			}
			if (i == 20)
			{
				//couldn't find the ')' character
				checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
			}
		}
		else if (buffer[0] == '0')
		{
			length = 0;
		}
		else
		{
			sscanf(buffer, "%lld", &length);
			if (length > 9 || length < 1)
			{
				//expected number 1 to 9
				checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
			}

			checkErr(viRead(instrSession, (ViBuf)buffer, (ViUInt32) length, NULL));
			buffer[length] = '\0';
			sscanf(buffer, "%lld", &length);
		}

		if (length > 0)
		{
			viCheckErr(Rs_ReadToFile2(instrSession, destination, length, RS_VAL_TRUNCATE));
			/* Flush garbage */
			viScanf(instrSession, "%*[^\r\n]");
			if (error > 0) error = VI_SUCCESS;
		}
		else
		{
			/* Unlimited size of data */
			checkErr(Rs_ReadDataUnknownLength(instrSession, &pbuffer, &ret_count));
		}
    }
    else
    {
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);         
    }
	
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);        
    checkErr( rsrtm20xx_CheckStatus (instrSession));
  
Error:
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    if (pbuffer) free(pbuffer);
    switch (intfc)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }  
    (void) Rs_UnlockSession(instrSession, NULL);
    
    return error;
}

/// HIFN  This function is used to read data from the host computer and write it 
/// HIFN  to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control computer 
/// HIPAR source/to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer from 
/// HIPAR destination/control computer to instrument applies.
ViStatus _VI_FUNC rsrtm20xx_WriteFromFileToInstrument(
    ViSession   instrSession,
    ViString    source,
    ViString    destination
)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt32    digits = 0;
    FILE        *sourceFile = NULL;
    ViUInt64    fileSize = 0;
#if defined (_CVI_)
	ViUInt32      fileSize32 = 0;
#endif
    
    checkErr( Rs_LockSession (instrSession, NULL));
    viCheckParm (!source, 2, "Source");
    viCheckParm (!destination, 3, "Destination");  
        
    /* Open source file from host controller */
    if ((sourceFile = fopen (source, "rb")) == NULL)
        viCheckErr(RS_ERROR_CANNOT_OPEN_FILE);

    /* Get size of file */
    (void) fseek (sourceFile, 0, SEEK_END);
#if defined (_CVI_)
    GetFileSize (source, &fileSize32);
	fileSize = (ViUInt64) fileSize32;
#elif defined(_MSC_VER)
    fileSize = (ViUInt64) _ftelli64 (sourceFile);
#else
    fileSize = (ViUInt64) ftell (sourceFile);
#endif
    (void) fseek (sourceFile, 0, SEEK_SET);

    (void) viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_FALSE);
		
    /* Create command header */
    digits = (ViUInt32) ((int)floor(log10((double)fileSize))) + 1;
    if (digits < 10)
	{
		viCheckErr (viPrintf (instrSession, ":MMEM:DATA '%s',#%ld%lld", destination, digits, fileSize));
	}
	else
	{
		viCheckErr (viPrintf (instrSession, ":MMEM:DATA '%s',#(%lld)", destination, fileSize));
	}
    
    viCheckErr (Rs_WriteFromFile2(instrSession, source, fileSize, 0));
	
    viCheckErr (viWrite(instrSession, (ViBuf) "\n", 1, NULL));
        
    checkErr( rsrtm20xx_CheckStatus (instrSession));
 
 Error:
    if (sourceFile) (void) fclose (sourceFile);
    
    (void) Rs_UnlockSession(instrSession, NULL);
    
    return error;
}

/// HIFN  This function allows to provide data set operation for Save/Recall 
/// HIFN  items of memory subsystem.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR stateNumber/Defines save and recall state position.
/// HIPAR path/This control specifies the source of the operation to be performed. 
ViStatus _VI_FUNC rsrtm20xx_DataSetFileOperations(
    ViSession   instrSession,
    ViInt32 operation,
    ViInt32 stateNumber,
    ViString    path
)
{
    ViStatus error = VI_SUCCESS;
    ViChar   buffer[RSRTM20XX_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    switch (operation)
    {
        case RSRTM20XX_VAL_MEM_SAVE:
            sprintf (buffer, "MMEM:STOR:STAT %ld,'%s'\n", stateNumber, path);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rsrtm20xx_CheckStatus (instrSession));
            break;                
        case RSRTM20XX_VAL_MEM_RECALL:
            sprintf (buffer, "MMEM:LOAD:STAT %ld,'%s'\n", stateNumber, path);
            checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
            checkErr (rsrtm20xx_CheckStatus (instrSession));
            break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Operation");
    }
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
ViStatus _VI_FUNC rsrtm20xx_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsrtm20xx_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/// HIFN This function performs all of the drivers clean-up operations
/// HIFN except for closing the RS session.  This function is called by
/// HIFN rsrtm20xx_close or by an RS class driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrtm20xx_init or 
/// HIPAR instrSession/rsrtm20xx_InitWithOptions function.
ViStatus _VI_FUNC rsrtm20xx_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

	checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_IO_SESSION, 0, (ViInt32) sizeof (io), &io));

Error:

    (void) Rs_SetAttribute (instrSession, NULL, RS_ATTR_IO_SESSION, 0, NULL);
	
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
